/***************************************************************************
                          Bomb.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Bomb.cpp,v 1.24 1999/05/15 20:14:25 dk Exp $

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "Bomb.h"
#include "ClanBomber.h"
#include "Explosion.h"
#include "Timer.h"

Bomb::Bomb( int _x, int _y, int _power, Bomber *_bomber, ClanBomberApplication *_app ) : GameObject( _x, _y, _app )
{
	detonation_countdown=2.5f;
	cur_scale = 0;
	cur_dir = DIR_NONE;
	exploding = false;
	power=_power;
	bomber = _bomber;
	bomber->dec_cur_bombs();
	app->objects.add( this );
	speed = 160;
	fields_to_fly = 1;
	can_kick = false;
	thrown = false;
}

Bomb::~Bomb()
{
	app->map->mapfield[(x+20)/40][(y+20)/40]->bomb=NULL;
}

void Bomb::stop()
{
	GameObject::stop();
	if (!flying)
	{
		cur_dir = DIR_NONE;
	}
}

void Bomb::show()
{
	if (exploding)
	{
		return;
	}
	
	if (!flying)
	{	if (falling)
		{
			app->sur_bomb->put_screen( x+80-fall_countdown*20, y+60-fall_countdown*20, fall_countdown,fall_countdown,(int)(cur_scale)%4 + bomber->get_color()*4 );		
		}
		else
		{
			app->sur_bomb->put_screen( x+60, y+40, (int)(cur_scale)%4 + bomber->get_color()*4 );
		}
	}
	else
	{
		float scalefactor = sinf(fly_power*3.14f*(4/(float)fields_to_fly)) * (fields_to_fly/5.0f);
		app->sur_bomb->put_screen( x+60-scalefactor*20.0f, y+40-scalefactor*60.0f, scalefactor+1.0f, scalefactor+1.0f,  bomber->get_color()*4);
	}
}

void Bomb::kick (Direction dir)
{
	if (!flying)
	{
		cur_dir = dir;
	}
}

void Bomb::throww(Direction dir, int fields)
{
	if (fields > 0)
	{
		flying = true;
		thrown = true;
		cur_dir = dir;
		fly_power = fields/4.0f;
		fields_to_fly = fields;
		app->map->mapfield[(x+20)/40][(y+20)/40]->bomb=NULL;	
	}
}

void Bomb::act()
{
	flying = false;
	GameObject::act();

	if (fallen_down)
	{
		delete_me= true;
		return;
	}
/*	if (falling)
	{
		return;
	}*/
	
	if (thrown)
	{
		flying = true;
		fly_power -= Timer::time_elapsed();
		if (fly_power<=0)
		{
			if (app->map->mapfield[(x+20)/40][(y+20)/40]->bomb != NULL)
			{
				fly_power = 0.25f;
				fields_to_fly = 1;
				snap();
			}
			else
			{
				fly_power = 0;
				flying = false;
				thrown = false;
				cur_dir= DIR_NONE;
				app->map->mapfield[(x+20)/40][(y+20)/40]->bomb = this;
				snap();
			}
		}
		move();
		z = Z_FLYING_BOMB + y;
	}
	else
	{
		MapField* old_map_field = app->map->mapfield[(x+20)/40][(y+20)/40];
		move();
		MapField* new_map_field = app->map->mapfield[(x+20)/40][(y+20)/40];
		cur_scale += Timer::time_elapsed()*10;

		if (new_map_field != old_map_field)
		{
			new_map_field->bomb = this;
			old_map_field->bomb = NULL;
		}
		detonation_countdown -= Timer::time_elapsed();
		if (detonation_countdown<=0)
		{
			explode();
		}
		z = Z_BOMB + y;
	}

	if (exploding)
	{
		detonation_countdown -= Timer::time_elapsed();
		if (detonation_countdown < -1.2f)
		{
			delete_me = true;
		}
	}
}

void Bomb::explode()
{
	if (!exploding && !falling)
	{
		snap();
		exploding = true;
		new Explosion( x, y, power, bomber, app );
		app->map->shake( (int)(0.35f/Timer::time_elapsed()) );
		bomber->inc_cur_bombs();
	}
}

void Bomb::explode_delayed()
{
	if (detonation_countdown > 0.1f)
	{
		detonation_countdown = 0.1f;
	}
}

int Bomb::get_power()
{
	return power;
}



