/***************************************************************************
                          Bomber.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Bomber.cpp,v 1.32 1999/05/15 16:00:08 dk Exp $

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "ClanBomber.h"
#include "Bomber.h"
#include "Bomber_Corpse.h"
#include "Bomb.h"
#include "Config.h"
#include "Timer.h"

#include "Extra_Bomb.h"
#include "Extra_Skateboard.h"
#include "Extra_Power.h"
#include "Extra_Kick.h"
#include "Extra_Glove.h"


Bomber::Bomber( int _x, int _y, COLOR _color, Controller* _controller, CL_String _name, ClanBomberApplication *_app) : GameObject( _x, _y, _app )
{
	name =_name;
	
	if (_x > 300)
	{
		cur_dir = DIR_LEFT;
	} else
	{
		cur_dir = DIR_RIGHT;
	}
	orig_x = _x;
	orig_y = _y;

	color = _color;
	controller = _controller;
	controller->attach(this);

	kills = 0;
	deaths = 0;
	points = 0;
	can_pass_bomber = true;
	can_throw = true;
	init();

	app->bomber_objects.add(this);
}


CL_String Bomber::get_name()
{
	return name;
}



Bomber::~Bomber()
{
	if (controller)
	{
		delete controller;
	}
}

void Bomber::show()
{
	if (cur_dir != DIR_NONE)
	{	
		if (falling)
		{
			app->sur_bomber[color]->put_screen( x+80-fall_countdown*20, y+41-fall_countdown*30, fall_countdown, fall_countdown, cur_dir*10 + (int)anim_count );
		}
		else
		{
			app->sur_bomber[color]->put_screen( x+60, y+11 + 9*(color>3), cur_dir*10 + (int)anim_count );
		}
	}
}

void Bomber::show (int _x, int _y)
{
	if (!dead)
	{
		app->sur_bomber[color]->put_screen( _x, _y, cur_dir*10 + (int)anim_count );
	}
	else
	{
		app->sur_bomber[color]->put_screen( _x, _y, cur_dir*10 + 9 );
	}
}

void Bomber::show_won()
{
	app->sur_bomber[color]->put_screen( 300, 300, 4.0f, 4.0f, 0 );
}

void Bomber::act()
{
	controller->update();
	
	GameObject::act();
	bool moved = false;

	if (dead)
	{
		return;
	}
	
	if (fallen_down)
	{
		die();
		return;
	}
		
	if (controller->is_left())
	{
		moved = true;
		anim_count += Timer::time_elapsed()*20*(speed/90);
		cur_dir = DIR_LEFT;
		move();
	} else
	if (controller->is_up())
	{
		moved = true;
		anim_count += Timer::time_elapsed()*20*(speed/90);
		cur_dir = DIR_UP;
		move();
	} else
	if (controller->is_right())
	{
		moved = true;
		anim_count += Timer::time_elapsed()*20*(speed/90);
		cur_dir = DIR_RIGHT;
		move();
	} else
	if (controller->is_down())
	{
		moved = true;
		anim_count += Timer::time_elapsed()*20*(speed/90);
		cur_dir = DIR_DOWN;
		move();
	} else
	{
		anim_count = 0;
	}
	if (anim_count >= 9) anim_count = 1;
	
	if ( controller->is_bomb() && !bomb_key)
	{
		if (app->map->mapfield[(x+20)/40][(y+20)/40]->bomb)
		{
			if (gloves && !moved)
			{
				app->map->mapfield[(x+20)/40][(y+20)/40]->bomb->throww(cur_dir);
				anim_count = 2;
			}
		}
		else
		{
			put_bomb();
		}
		bomb_key = true;
	} else
	if ( !(controller->is_bomb()) )
	{
		bomb_key = false;
	}
	if (!falling)
	{
		z = Z_BOMBER + y;
	}
}

void Bomber::put_bomb()
{
	if (cur_bombs && !falling)
	{
		if (app->map->add_bomb( (x+20)/40, (y+20)/40, power, this ))
		{
			app->sfx_putbomb->play();
		}
	}
}

void Bomber::gain_extra_skateboard()
{
	if (Config::get_max_skateboards() > skateboards)
	{
		skateboards++;
		extra_skateboards++;
		speed+=40;
	}
}


void Bomber::gain_extra_bomb()
{
	if (Config::get_max_bombs() > bombs)
	{
		bombs++;
		extra_bombs++;
		cur_bombs++;
	}
}

void Bomber::gain_extra_glove()
{
	gloves++;
	extra_gloves++;
}

Bomber::COLOR Bomber::get_color()
{
	return color;
}

void Bomber::gain_extra_power()
{
	if (Config::get_max_power() > power)
	{
		power++;
		extra_power++;
	}
}

void Bomber::loose_all_extras()
{
	int i;
	for (i=0;i<extra_bombs;i++)
	{
		new Extra_Bomb(app,x,y);
		
	}
	extra_bombs = 0;

	for (i=0;i<extra_power;i++)
	{
		new Extra_Power(app,x,y);
	}
	extra_power = 0;

	for (i=0;i<extra_skateboards;i++)
	{
		new Extra_Skateboard(app,x,y);
	}
	extra_skateboards = 0;

	for (i=0;i<extra_kick;i++)
	{
		new Extra_Kick(app,x,y);
	}
	extra_kick = 0;

	for (i=0;i<extra_gloves;i++)
	{
		new Extra_Glove(app,x,y);
	}
	extra_gloves = 0;
}

void Bomber::gain_extra_kick()
{
		can_kick = true;
		extra_kick++;
}

void Bomber::inc_cur_bombs()
{
	cur_bombs++;
}

void Bomber::dec_cur_bombs()
{
	if (cur_bombs)
	{
		cur_bombs--;
	}
}


void Bomber::init()
{
	dead = false;
	falling = false;
	fallen_down = false;

	extra_gloves = 0;
	extra_skateboards =0;
	extra_power = 0;
	extra_kick = 0;
	extra_bombs = 0;

	skateboards = Config::get_start_skateboards();
	speed = 90 + (skateboards*40);
	can_kick = Config::get_start_kick();
	cur_bombs = Config::get_start_bombs();
	bombs = Config::get_start_bombs();
	power = Config::get_start_power();
	gloves = Config::get_start_glove();
	
	x = orig_x;
	y = orig_y;
	anim_count = 0;

	bomb_key = true;
	
	controller->reset();
}

bool Bomber::die()
{
	if (dead)
	{
		return false;
	}
	dead = true;
	deaths++;
	app->sfx_die->play();

	if (!fallen_down)
	{
		new Bomber_Corpse(x, y, color, cur_dir*10+9, app);
		loose_all_extras();
	}
	return true;
}

void Bomber::inc_kills()
{
	kills++;
}

int Bomber::get_cur_bombs()
{
	return cur_bombs;
}

int Bomber::get_kills()
{
	return kills;
}

int Bomber::get_points()
{
	return points;
}

void Bomber::set_points( int _points )
{
	points = _points;
}

void Bomber::inc_points()
{
	points++;
}

void Bomber::dec_points()
{
	points--;
}

bool Bomber::is_dead()
{
	return dead;
}

int Bomber::get_deaths()
{
	return deaths;
}









































































