/***************************************************************************
                          ClanBomber.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: ClanBomber.cpp,v 1.75 1999/05/15 15:56:45 dk Exp $


#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "ClanBomber.h"
#include "Controller.h"
#include "Debug.h"
#include "Observer_Classic.h"
#include "Config.h"
#include "Menu.h"
#include "Timer.h"
#include "PlayerSetup.h"
#include <config.h>

#include <time.h>

ClanBomberApplication app;

char *ClanBomberApplication::get_title()
{
	return "ClanBomber, the cool game that uses ClanLib";
}

int ClanBomberApplication::main(int argc, char** argv)
{
	srand( (long)time(NULL) );

	CL_System::init_display();
	CL_System::init_sound();

	CL_Display::set_videomode(800, 600, 16);

	CL_Display::clear_display();
	CL_Display::flip_display();
	CL_Display::clear_display();
	CL_Display::flip_display();

#ifndef WIN32
	Config::set_filename( CL_String(getenv("HOME")) + "/.clanbomber.cfg" );
#endif
	
	res_game = NULL;
	res_app = new CL_ResourceManager("clanbomber.scr");

	CL_Palette *pal = CL_Res_Palette::load("App/pal_fnt", *res_app);
	CL_Display::set_palette(pal);
	delete pal;
	fnt_clansoft = CL_Res_Font::load("App/fnt_clansoft", *res_app);
	
	run_intro();
	
	load_resources();
	
	
	Config::load();
	
	CL_Array<CL_String> controller_strings;
	controller_strings.add( new CL_String("none") );
	controller_strings.add( new CL_String("AI") );
	controller_strings.add( new CL_String("Keymap 1") );
	controller_strings.add( new CL_String("Keymap 2") );
	controller_strings.add( new CL_String("Keymap 3") );
	if (CL_Input::joysticks.get_num_items() > 0)
	{
		controller_strings.add( new CL_String("Joystick 1") );
	}
	if (CL_Input::joysticks.get_num_items() > 1)
	{
		controller_strings.add( new CL_String("Joystick 2") );
	}
	
	CL_Array<CL_String> theme_names;
	theme_names.add( new CL_String("Jungle Arena") );
	theme_names.add( new CL_String("Blue Ground") );
	theme_names.add( new CL_String("Green Crypt") );
	theme_names.add( new CL_String("Cool Night") );
	
	menu = new Menu( CL_String("ClanBomber V")+VERSION, this);
	menu->add_item( CL_String("Local Game"), MENU_LOCALGAME );
		menu->add_item( CL_String("Start"), LOCALGAME_START, MENU_LOCALGAME );
		menu->add_item( CL_String("Player Setup"), MENU_PLAYER_SETUP, MENU_LOCALGAME);
/*			menu->add_string( CL_String("Player 1"), CONFIG_PLAYER1_NAME, MENU_PLAYER_SETUP, Config::get_player_name(0) );
			menu->add_stringlist( CL_String("Player 1"), CONFIG_CONTROLLER1, MENU_PLAYER_SETUP, controller_strings, Config::get_controller(0) );
			menu->add_string( CL_String("Player 2"), CONFIG_PLAYER2_NAME, MENU_PLAYER_SETUP, Config::get_player_name(1) );
			menu->add_stringlist( CL_String("Player 2"), CONFIG_CONTROLLER2, MENU_PLAYER_SETUP, controller_strings, Config::get_controller(1) );
			menu->add_string( CL_String("Player 3"), CONFIG_PLAYER3_NAME, MENU_PLAYER_SETUP, Config::get_player_name(2) );
			menu->add_stringlist( CL_String("Player 3"), CONFIG_CONTROLLER3, MENU_PLAYER_SETUP, controller_strings, Config::get_controller(2) );
			menu->add_string( CL_String("Player 4"), CONFIG_PLAYER4_NAME, MENU_PLAYER_SETUP, Config::get_player_name(3) );
			menu->add_stringlist( CL_String("Player 4"), CONFIG_CONTROLLER4, MENU_PLAYER_SETUP, controller_strings, Config::get_controller(3) );*/

		menu->add_value( CL_String("Level"), CONFIG_STARTLEVEL, MENU_LOCALGAME, 1, MAP_COUNT, Config::get_start_level() );
		menu->add_value( CL_String("Points to win"), CONFIG_POINTS, MENU_LOCALGAME, 1, 10, Config::get_points_to_win() );
		menu->add_value( CL_String("Round Time"), CONFIG_ROUND_TIME, MENU_LOCALGAME, 30, 300, Config::get_round_time() );
//	menu->add_item( CL_String("xNetworkxGamex"), MENU_NETWORKGAME );
	menu->add_item( CL_String("Options"), MENU_OPTIONS );
		menu->add_item( CL_String("Extras"), MENU_EXTRAS, MENU_OPTIONS );
			menu->add_value( CL_String("Start Bombs"), CONFIG_BOMBS, MENU_EXTRAS, 1, 15, Config::get_start_bombs() );
			menu->add_value( CL_String("Start Power"), CONFIG_POWER, MENU_EXTRAS, 1, 15, Config::get_start_power() );
			menu->add_value( CL_String("Start Skateboards"), CONFIG_SKATES, MENU_EXTRAS, 0, 10, Config::get_start_skateboards() );
			menu->add_value( CL_String("Start Kick"), CONFIG_KICK, MENU_EXTRAS, 0, 1, Config::get_start_kick() );
			menu->add_value( CL_String("Start Glove"), CONFIG_GLOVE, MENU_EXTRAS, 0, 1, Config::get_start_glove() );
			menu->add_value( CL_String("Max. Bombs"), CONFIG_MAX_BOMBS, MENU_EXTRAS, 1, 15, Config::get_max_bombs() );
			menu->add_value( CL_String("Max. Power"), CONFIG_MAX_POWER, MENU_EXTRAS, 1, 15, Config::get_max_power() );
			menu->add_value( CL_String("Max. Skateboards"), CONFIG_MAX_SKATES, MENU_EXTRAS, 0, 10, Config::get_max_skateboards() );
		menu->add_stringlist( CL_String("Theme"), CONFIG_THEME, MENU_OPTIONS, theme_names, Config::get_theme() );
		menu->add_value( CL_String("Rotate Levels"), CONFIG_ROTATE_LEVELS, MENU_OPTIONS, 0, 1, Config::get_rotate_levels() );
	menu->add_item( CL_String("Credits"), MENU_CREDITS );
	menu->add_item( CL_String("Help"), MENU_HELP );
	menu->add_item( CL_String("Exit"), MENU_EXIT );

	menu->scroll_in();
	
	while (1)
	{
		int result = menu->execute();
		MenuItem* item = menu->get_item_by_id( result );
		switch (result)
		{
			case MENU_EXIT:
				menu->scroll_out();
				delete menu;
				return 0;
			break;
			case MENU_PLAYER_SETUP:
				PlayerSetup* ps;
				ps = new PlayerSetup(this);
				ps->run();
				delete ps;
			break;
			case CONFIG_BOMBS:
				Config::set_start_bombs( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_POWER:
				Config::set_start_power( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_SKATES:
				Config::set_start_skateboards( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_KICK:
				Config::set_start_kick( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_GLOVE:
				Config::set_start_glove( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_ROTATE_LEVELS:
				Config::set_rotate_levels( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_MAX_BOMBS:
				Config::set_max_bombs( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_MAX_POWER:
				Config::set_max_power( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_MAX_SKATES:
				Config::set_max_skateboards( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case MENU_CREDITS:
				show_credits();				
			break;
			case MENU_HELP:
				show_tutorial();				
			break;
			case CONFIG_STARTLEVEL:
				Config::set_start_level( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_POINTS:
				Config::set_points_to_win( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;
			case CONFIG_ROUND_TIME:
				if (((MenuItem_Value*)item)->get_value()%30 == 1)
				{
					((MenuItem_Value*)item)->set_value( ((MenuItem_Value*)item)->get_value() + 29 );
				} else
				if (((MenuItem_Value*)item)->get_value()%30 == 29)
				{
					((MenuItem_Value*)item)->set_value( ((MenuItem_Value*)item)->get_value() - 29 );
				}
				Config::set_round_time( ((MenuItem_Value*)item)->get_value() );
				Config::save();
			break;			
			case CONFIG_THEME:
				Config::set_theme( ((MenuItem_StringList*)item)->get_value() );
				Config::save();
			break;
			case LOCALGAME_START:
				menu->scroll_out();
				init_game();
				run_game();
				deinit_game();
				menu->scroll_in();
			break;
			default:
				cout << result << endl;
			break;
		}
	}

	return 1; // shouldnt reach this
}

void ClanBomberApplication::run_game()
{
	Timer::reset();
	
	CL_Display::clear_display();
	CL_Display::flip_display();
	CL_Display::clear_display();
	CL_Display::flip_display();

	sfx_furz->play();
	
	while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE) == 0)
	{
		Timer::time_elapsed(true);

		act_all();
		observer->act();
		
		if (observer->end_of_game_requested())
		{
			break;
		}
		
		show_all();
		observer->show();
		
		CL_Display::flip_display();
		
		if (CL_System::keep_alive())
		{
			break;
		}
	}
}

void ClanBomberApplication::act_all()
{
	map->act();

	CL_Iterator<GameObject> object_counter(objects);
	while (object_counter.next() != NULL)
	{
		object_counter()->act();
		if (object_counter()->delete_me)
		{
			delete object_counter();
			object_counter.remove();
		}
	}
	CL_Iterator<Bomber> bomber_object_counter(bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{
		bomber_object_counter()->act();
		if (bomber_object_counter()->delete_me)
		{
			delete bomber_object_counter();
			bomber_object_counter.remove();
		}
	}
}

void ClanBomberApplication::show_all()
{
	
	GameObject* draw_list[objects.get_num_items()+bomber_objects.get_num_items()];
	int n = 0;
	int i;
	GameObject* obj;

	CL_Iterator<GameObject> object_counter(objects);
	while (object_counter.next() != NULL)
	{
		draw_list[n] = object_counter();
		n++;
	}
	CL_Iterator<Bomber> bomber_object_counter(bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{
		draw_list[n] = bomber_object_counter();
		n++;
	}
	
	bool sort = true;
	while(sort)
	{
		sort = false;
		for( i=0; i<n-1; i++ )
		{
			if (draw_list[i]->get_z() > draw_list[i+1]->get_z())
			{
				obj = draw_list[i];
				draw_list[i] = draw_list[i+1];
				draw_list[i+1] = obj;
				sort = true;
			}
		}
	}

	map->refresh_holes();
	bool drawn_map = false;
	
	for( i=0; i<n; i++ )
	{
		if (draw_list[i]->get_z()>=Z_GROUND && drawn_map == false)
		{
			map->show();
			drawn_map= true;
		}
		draw_list[i]->show();
	}
}

void ClanBomberApplication::load_resources()
{
	CL_Display::clear_display();
	fnt_clansoft->print_center( 400, 300, "Loading..." );
	CL_Display::flip_display();
		
	res_game = new CL_ResourceManager("clanbomber.scr");
	
	// init graphics
	sur_maptiles = CL_Res_Surface::load("Game/sur_maptiles", *res_game);
	sur_extras = CL_Res_Surface::load("Game/sur_extras", *res_game);
	sur_bomber[0] = CL_Res_Surface::load("Game/sur_bomber_red", *res_game);
	sur_bomber[1] = CL_Res_Surface::load("Game/sur_bomber_lila", *res_game);
	sur_bomber[2] = CL_Res_Surface::load("Game/sur_bomber_green", *res_game);
	sur_bomber[3] = CL_Res_Surface::load("Game/sur_bomber_blue", *res_game);
	sur_bomber[4] = CL_Res_Surface::load("Game/sur_bomber_d_red", *res_game);
	sur_bomber[5] = CL_Res_Surface::load("Game/sur_bomber_d_lila", *res_game);
	sur_bomber[6] = CL_Res_Surface::load("Game/sur_bomber_d_green", *res_game);
	sur_bomber[7] = CL_Res_Surface::load("Game/sur_bomber_d_blue", *res_game);
	
	sur_observer = CL_Res_Surface::load("Game/sur_observer", *res_game);
	sur_bomb = CL_Res_Surface::load("Game/sur_bomb", *res_game);
	sur_explode = CL_Res_Surface::load("Game/sur_explode", *res_game);
	sur_ps_teams = CL_Res_Surface::load("Game/sur_ps_teams", *res_game);
	sur_ps_controls = CL_Res_Surface::load("Game/sur_ps_controls", *res_game);


	// init sounds
	sfx_putbomb = CL_Res_Sample::load("Sounds/sfx_putbomb", *res_game);
	sfx_explode = CL_Res_Sample::load("Sounds/sfx_explode", *res_game);
	sfx_die = CL_Res_Sample::load("Sounds/sfx_die", *res_game);
	sfx_hurryup = CL_Res_Sample::load("Sounds/sfx_hurryup", *res_game);
	sfx_hurryup->set_volume(0.8f);
	sfx_wow = CL_Res_Sample::load("Sounds/sfx_wow", *res_game);
	sfx_furz = CL_Res_Sample::load("Sounds/sfx_furz", *res_game);
	sfx_splash[0] = CL_Res_Sample::load("Splash/spl_1", *res_game);
	sfx_splash[1] = CL_Res_Sample::load("Splash/spl_2", *res_game);
	sfx_splash[2] = CL_Res_Sample::load("Splash/spl_3", *res_game);
	sfx_splash[3] = CL_Res_Sample::load("Splash/spl_4", *res_game);
	sfx_splash[4] = CL_Res_Sample::load("Splash/spl_5", *res_game);
	sfx_splash[5] = CL_Res_Sample::load("Splash/spl_6", *res_game);
	sfx_splash[6] = CL_Res_Sample::load("Splash/spl_7", *res_game);
	sfx_splash[7] = CL_Res_Sample::load("Splash/spl_8", *res_game);
	sfx_splash[8] = CL_Res_Sample::load("Splash/spl_9", *res_game);
	sfx_splash[9] = CL_Res_Sample::load("Splash/spl_10", *res_game);
	sfx_splash[10] = CL_Res_Sample::load("Splash/spl_1a", *res_game);
	sfx_splash[11] = CL_Res_Sample::load("Splash/spl_2a", *res_game);

//	wav = CL_Streamed_RawSample::create("!mpg123 -q --aggressive -s /usr/local/mp3/DIMMU\\ BORGIR\\ -\\ Spiritual\\ Black\\ Dimensions/track04.mp3", sf_16bit_signed_stereo, 44100, NULL, false);
//	wav = CL_Streamed_WaveSample::create("!mpg123 --aggressive -w - /usr/local/mp3/DIMMU\\ BORGIR\\ -\\ Spiritual\\ Black\\ Dimensions/track04.mp3", NULL, false);
//	wav = CL_Streamed_WaveSample::create("/home/dk/wav.wav", NULL, true);
#ifdef MUSIC	
	wav = CL_Streamed_RawSample::create("!mikmod -q -d 4 /home/dk/Clan/tracks/bud.mod", sf_16bit_signed_stereo, 44100, NULL, true);
	ses = wav->prepare();
#endif
}

void ClanBomberApplication::init_game()
{
#ifdef MUSIC
	ses.play();
#endif
		
	// init map
	map = new Map(this);
	map->load(CL_String("Levels/level")+Config::get_start_level());

	// init GameObjects
	CL_Vector pos;

	for (int i=0; i< 8 ; i++)
	{
		if (Config::bomber[i].is_enabled())
		{
			pos = map->get_bomber_pos(i);
		     new Bomber( pos.x*40, pos.y*40, (Bomber::COLOR)(Config::bomber[i].get_skin()), Controller::create((Controller::CONTROLLER_TYPE)Config::bomber[i].get_controller()), Config::bomber[i].get_name(), this );
		}		
	}

	new Debug( 0, 0, this );
	observer = new Observer_Classic( 0, 0, this );
}

void ClanBomberApplication::deinit_game()
{
	// delete all GameObjects
	CL_Iterator<GameObject> object_counter(objects);
	while (object_counter.next() != NULL)
	{
		delete object_counter();
	}
	objects.empty_list();
	
	CL_Iterator<Bomber> bomber_object_counter(bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{
		delete bomber_object_counter();
	}
	bomber_objects.empty_list();
	
	// delete map
	delete map;
	delete observer;
	
#ifdef MUSIC
	ses.stop();
	ses = wav->prepare();
#endif
}

void ClanBomberApplication::run_intro()
{
	fnt_smallfont = CL_Res_Font::load("App/fnt_smallfont", *res_app);
	sfx_typewriter = CL_Res_Sample::load("Sounds/sfx_typewriter",*res_app);
	sur_fl_logo = CL_Res_Surface::load("App/sur_fl_logo", *res_app);
	Timer::reset();
	float alpha = 1;
	while(1)
	{
		CL_Display::clear_display();
		CL_System::keep_alive();
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
		{
			return;
		}
		if (alpha > 0)
		{
			sur_fl_logo->put_screen( 100, 250, 0 );
			CL_Display::fill_rect( 100, 250, 700, 356, 0, 0, 0, alpha );
			CL_Display::flip_display();
	
			alpha -= Timer::time_elapsed(true) / alpha;
		}
		else
		{
			sur_fl_logo->put_screen( 100, 250, 0 );
			CL_Display::flip_display();
			break;
		}
	}
		
	CL_String *domi_str = new CL_String("A WORLD DOMINATION PROJECT");
	for (int domispell=0;domispell<=domi_str->get_length();domispell++)
	{
		CL_Display::clear_display();
		CL_System::keep_alive();
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
		{
			return;
		}
		if (domi_str->mid(domispell,1) != " ")
		{
			sfx_typewriter->play();
		}
		sur_fl_logo->put_screen( 100, 250, 0 );
//		fnt_smallfont->print_left( 230, 360, domi_str->mid(0,domispell) );
		fnt_smallfont->print_center( 400, 360, domi_str->mid(0,domispell) );
		CL_Display::flip_display();
		CL_System::sleep(rand()%100 + 80);
	}
	CL_System::sleep(1500);
	
	Timer::reset();
	float scroll = 100;
	while(scroll<800)
	{
		CL_Display::clear_display();
		CL_System::keep_alive();
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
		{
			return;
		}
		sur_fl_logo->put_screen( (int)scroll, 250,0);
		fnt_smallfont->print_center( 400, 360, *domi_str );
		
		CL_Display::flip_display();
		
		scroll += Timer::time_elapsed(true)*1100.0f;
	}
}

void ClanBomberApplication::show_credits()
{
	bool key_esc = false;
	float x = 0;
	float scroll = 500;
	bool andi_will_auch_nach_vorne;
	andi_will_auch_nach_vorne = rand()%2;
	
	Timer::reset();
	while (1)
	{
		CL_Display::clear_display();

		fnt_clansoft->print_center( rand()%800, rand()%600+scroll, ">" );
		fnt_clansoft->print_center( rand()%800, rand()%600+scroll, "o" );
		fnt_clansoft->print_center( rand()%800, rand()%600+scroll, "." );

		fnt_clansoft->print_center( 400 + sin(x/2.6f)*50, 80+scroll + cos(x)*15, "ClanBomber Credits" );

		fnt_clansoft->print_center( 400 + sin(x/2.6f+0.1f)*60, 180+scroll + sin(x)*15, "Coders" );
		if (andi_will_auch_nach_vorne)
		{
			fnt_clansoft->print_center( 400 + sin(x/2.6f+0.2f)*80, 230+scroll + sin(x)*15, "Andreas Hundt" );
			fnt_clansoft->print_center( 400 + sin(x/2.6f+0.3f)*100, 270+scroll + sin(x)*15, "Denis Oliver Kropp" );
		}
		else
		{
			fnt_clansoft->print_center( 400 + sin(x/2.6f+0.2f)*80, 230+scroll + sin(x)*15, "Denis Oliver Kropp" );
			fnt_clansoft->print_center( 400 + sin(x/2.6f+0.3f)*100, 270+scroll + sin(x)*15, "Andreas Hundt" );
		}

		fnt_clansoft->print_center( 400 + sin(x/2.6f+0.5f)*60, 370+scroll + sin(x)*15, "Graphics" );
		fnt_clansoft->print_center( 400 + sin(x/2.6f+0.6f)*80, 420+scroll + sin(x)*15, "Denis Oliver Kropp" );
		fnt_clansoft->print_center( 400 + sin(x/2.6f+0.7f)*100, 460+scroll + sin(x)*15, "Andreas Hundt" );
		fnt_clansoft->print_center( 400 + sin(x/2.6f+0.8f)*120, 500+scroll + sin(x)*15, "Jens mach mal was" );

		CL_Display::flip_display();
		CL_System::keep_alive();
		
		x += Timer::time_elapsed(true)*7.0f;
		if (scroll > 0)
		{
			scroll -= Timer::time_elapsed() * scroll * 0.15f;
		}

		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE) && !key_esc)
		{
			key_esc = true;
		} else
		if (!CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
		{
			if (key_esc)
			{
				return;
			}
		}
	}
}

void ClanBomberApplication::show_tutorial()
{
	CL_Display::clear_display();
	CL_Display::flip_display();
	int y = 25;
		
	CL_Display::clear_display();
	fnt_clansoft->print_center(400,y, "ClanBomber Extras");
	CL_Display::flip_display();
	fnt_clansoft->print_center(400,y, "ClanBomber Extras");
	CL_System::sleep(1000);
			
	y+=80;	
	sur_extras->put_screen( 15,y-5,0);
	fnt_clansoft->print_left(70,y, "Bomb:");
	fnt_clansoft->print_left(250,y, "You can place an additional bomb");
	CL_Display::flip_display();
	sur_extras->put_screen( 15,y-5,0);
	fnt_clansoft->print_left(70,y, "Bomb:");
	fnt_clansoft->print_left(250,y, "You can place an additional bomb");
	CL_System::sleep(1000);


	y+=50;
	sur_extras->put_screen( 15,y-5,1);
	fnt_clansoft->print_left(70,y, "Power:");
	fnt_clansoft->print_left(250,y, "Explosions grow one field in each direction");
	CL_Display::flip_display();
	sur_extras->put_screen( 15,y-5,1);
	fnt_clansoft->print_left(70,y, "Power:");
	fnt_clansoft->print_left(250,y, "Explosions grow one field in each direction");
	CL_System::sleep(1000);

	y+=50;
	sur_extras->put_screen( 15,y-5,2);
	fnt_clansoft->print_left(70,y, "Skateboard:");
	fnt_clansoft->print_left(250,y, "Lets you move faster");
	CL_Display::flip_display();
	sur_extras->put_screen( 15,y-5,2);	
	fnt_clansoft->print_left(70,y, "Skateboard:");
	fnt_clansoft->print_left(250,y, "Lets you move faster");
	CL_System::sleep(1000);

	y+=50;
	sur_extras->put_screen( 15,y-5,3);
	fnt_clansoft->print_left(70,y, "Kick:");
	fnt_clansoft->print_left(250,y, "Kick bombs if you walk against one");
	CL_Display::flip_display();
	sur_extras->put_screen( 15,y-5,3);	
	fnt_clansoft->print_left(70,y, "Kick:");
	fnt_clansoft->print_left(250,y, "Kick bombs if you walk against one");
	CL_System::sleep(1000);
	
	y+=50;
	sur_extras->put_screen( 15,y-5,4);
	fnt_clansoft->print_left(70,y, "Throw:");
	fnt_clansoft->print_left(250,y, "Throw Bombs if you press the button twice");
	fnt_clansoft->print_left(250,y+50, "without moving");
	CL_Display::flip_display();
	sur_extras->put_screen( 15,y-5,4);	
	fnt_clansoft->print_left(70,y, "Throw:");
	fnt_clansoft->print_left(250,y, "Throw Bombs if you press the button twice");
	fnt_clansoft->print_left(250,y+50, "without moving");
	CL_System::sleep(1000);
	
	y+=180;
	fnt_clansoft->print_center(400,y, "Press Escape");
	CL_Display::flip_display();		
	
	while (!CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
	{
		CL_System::keep_alive();
	}
}











