/***************************************************************************
                          Config.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Config.cpp,v 1.21 1999/05/15 15:56:45 dk Exp $

#include <ClanLib/clanlib.h>
#include "Config.h"
#include "Controller.h"
#include "fstream.h"
#include "stdio.h"

CL_String Config::filename	= "clanbomber.cfg";

int  Config::round_time       = 90;

int  Config::max_skateboards	= 7;
int  Config::max_power		= 8;
int  Config::max_bombs		= 8;

int  Config::start_skateboards= 0;
int  Config::start_power		= 1;
int  Config::start_bombs		= 1;
bool Config::start_kick		= false;
bool Config::start_glove		= false;

int  Config::start_level		= 1;
bool Config::rotate_levels	= false;

int  Config::points_to_win	= 5;
int  Config::theme			= 0;

BomberConfig Config::bomber[8];


BomberConfig::BomberConfig()
{
	enabled = true;
	team = 0;
	skin = 0;
	controller = 1;
	name = "Fischlustig";
}

BomberConfig::~BomberConfig()
{
}

void BomberConfig::set_name(CL_String _name)
{
	name = _name;
}

CL_String BomberConfig::get_name()
{
	return name;
}

void BomberConfig::set_skin(int _skin)
{
	skin = _skin%8;
}

int BomberConfig::get_skin()
{
	return skin;
}

void BomberConfig::set_team(int _team)
{
	team = _team%5;
}

int BomberConfig::get_team()
{
	return team;
}

void BomberConfig::set_controller(int _controller)
{
	controller = _controller% (4 + CL_Input::joysticks.get_num_items());
}

int BomberConfig::get_controller()
{
	return controller;
}

void BomberConfig::enable()
{
	enabled = true;
}

void BomberConfig::disable()
{
	enabled = false;
}

bool BomberConfig::is_enabled()
{
	return enabled;
}

void BomberConfig::set_enabled(bool _enabled)
{
	enabled = _enabled;
}


void Config::set_round_time( int _round_time)
{
	round_time = _round_time;
}

void Config::set_max_skateboards(int _max_skateboards)
{
	max_skateboards = _max_skateboards;
}

void Config::set_max_power(int _max_power)
{
	max_power = _max_power;
}

void Config::set_max_bombs(int _max_bombs)
{
	max_bombs = _max_bombs;
}

void Config::set_start_skateboards(int _start_skateboards)
{
	start_skateboards = _start_skateboards;
}


void Config::set_start_power(int _start_power)
{
	start_power = _start_power;
}


void Config::set_start_bombs(int _start_bombs)
{
	start_bombs = _start_bombs;
}

void Config::set_start_kick( bool _start_kick)
{
	start_kick = _start_kick;
}

void Config::set_start_glove( bool _start_glove)
{
	start_glove = _start_glove;
}

void Config::set_start_level(int _start_level)
{
	start_level = _start_level;
}

void Config::set_rotate_levels( bool _rotate_levels)
{
	rotate_levels = _rotate_levels;
}
	
void Config::set_points_to_win(int _points_to_win)
{
	points_to_win = _points_to_win;
}

void Config::set_theme(int _theme)
{
	theme = _theme;
}

void Config::set_filename(CL_String _filename)
{
	filename = _filename;
}

int Config::get_max_skateboards()
{
	return max_skateboards;
}

int Config::get_max_power()
{
	return max_power;
}

int Config::get_max_bombs()
{
	return max_bombs;
}

int Config::get_start_skateboards()
{
	return start_skateboards;
}

int Config::get_start_power()
{
	return start_power;
}

int Config::get_start_bombs()
{
	return start_bombs;
}

bool Config::get_start_kick()
{
	return start_kick;
}

bool Config::get_start_glove()
{
	return start_glove;
}

int Config::get_start_level()
{
	return start_level;
}

bool Config::get_rotate_levels()
{
	return rotate_levels;
}

int Config::get_round_time()
{
	return round_time;
}

int Config::get_points_to_win()
{
	return points_to_win;
}

int Config::get_theme()
{
  return theme;
}

bool Config::save()
{
	ofstream configfile(filename);

	configfile << "13" << endl; // version

	configfile << max_bombs			<< endl;
	configfile << max_power			<< endl;
	configfile << max_skateboards		<< endl;
	configfile << start_bombs		<< endl;
	configfile << start_power		<< endl;
	configfile << start_skateboards	<< endl;
	configfile << start_kick			<< endl;
	configfile << start_glove		<< endl;
	configfile << start_level		<< endl;
	configfile << rotate_levels		<< endl;
	configfile << points_to_win		<< endl;
	configfile << round_time			<< endl;
	configfile << theme				<< endl;
	
	for (int i=0; i<8; i++)
	{
		configfile << bomber[i].get_skin()			<< endl;
		configfile << bomber[i].get_name()			<< endl;
		configfile << bomber[i].get_team()			<< endl;
		configfile << bomber[i].get_controller()	<< endl;
		configfile << bomber[i].is_enabled()		<< endl;
	}

	return true;
}

bool Config::load()
{
	FILE* configfile = fopen( filename, "r" );

	if (!configfile)
	{
		save();
		return false;
	}

	int version;
	fscanf( configfile, "%d\n", &version );

	if (version != 13)
	{
		fclose( configfile );
		save();
		return false;
	}

	fscanf( configfile, "%d\n", &max_bombs );
	fscanf( configfile, "%d\n", &max_power );
	fscanf( configfile, "%d\n", &max_skateboards );
	fscanf( configfile, "%d\n", &start_bombs );
	fscanf( configfile, "%d\n", &start_power );
	fscanf( configfile, "%d\n", &start_skateboards );
	fscanf( configfile, "%d\n", (int*)&start_kick );
	fscanf( configfile, "%d\n", (int*)&start_glove );
	fscanf( configfile, "%d\n", &start_level );
	fscanf( configfile, "%d\n", (int*)&rotate_levels );
	fscanf( configfile, "%d\n", &points_to_win );
	fscanf( configfile, "%d\n", &round_time );
	fscanf( configfile, "%d\n", &theme );
	
	for (int i=0; i<8; i++)
	{
		fscanf( configfile, "%d\n", &version );
		bomber[i].set_skin(version);
		
		char c[2] = {0,0};
		CL_String player_name = "";
		while ((c[0] = fgetc(configfile)) != 10)
		{
			player_name += c;
		}
		bomber[i].set_name(player_name);
		
		fscanf( configfile, "%d\n", &version );
		bomber[i].set_team(version);
		
		fscanf( configfile, "%d\n", &version );
		bomber[i].set_controller(version);
	
		fscanf( configfile, "%d\n", &version );
		bomber[i].set_enabled((bool)version);
	}
	
	fclose( configfile );

	return true;
}















