/***************************************************************************
                          Controller.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Controller.cpp,v 1.10 1999/05/15 15:56:45 dk Exp $

#include <ClanLib/clanlib.h>

#include "Controller.h"
#include "Controller_AI.h"
#include "Bomber.h"

Controller::Controller()
{
	bomber = NULL;
}

void Controller::attach(Bomber* _bomber)
{
	bomber = _bomber;
}

/* Keyboard Controller */

Controller* Controller::create( CONTROLLER_TYPE _type )
{
	switch( _type )
	{
		case KEYMAP_1:
			return new Controller_Keyboard(0);
		break;
		case KEYMAP_2:
			return new Controller_Keyboard(1);
		break;
		case KEYMAP_3:
			return new Controller_Keyboard(2);
		break;
		case JOYSTICK_1:
			return new Controller_Joystick(0);
		break;
		case JOYSTICK_2:
			return new Controller_Joystick(1);
		break;
		case JOYSTICK_3:
			return new Controller_Joystick(2);
		break;
		case AI:
			return new Controller_AI();
		break;
		default:
		break;
	}
	
	return NULL; // whats up?
}

Controller_Keyboard::Controller_Keyboard(int keymap_nr) : Controller()
{
	switch (keymap_nr)
	{
	case 0:
		left_key = CL_Keyboard::KEY_LEFT;
		right_key = CL_Keyboard::KEY_RIGHT;
		up_key = CL_Keyboard::KEY_UP;
		down_key = CL_Keyboard::KEY_DOWN;
		bomb_key = CL_Keyboard::KEY_ENTER;
		break;
	case 1:
		left_key = CL_Keyboard::KEY_A;
		right_key = CL_Keyboard::KEY_D;
		up_key = CL_Keyboard::KEY_W;
		down_key = CL_Keyboard::KEY_S;
		bomb_key = CL_Keyboard::KEY_TAB;
		break;
	case 2:
		left_key = CL_Keyboard::KEY_J;
		right_key = CL_Keyboard::KEY_L;
		up_key = CL_Keyboard::KEY_I;
		down_key = CL_Keyboard::KEY_K;
		bomb_key = CL_Keyboard::KEY_SPACE;
		break;
	case 3:
		left_key = CL_Keyboard::KEY_4;
		right_key = CL_Keyboard::KEY_6;
		up_key = CL_Keyboard::KEY_8;
		down_key = CL_Keyboard::KEY_5;
		bomb_key = CL_Keyboard::KEY_0;
		break;
	}
}

void Controller_Keyboard::update()
{
}

void Controller_Keyboard::reset()
{
}

bool Controller_Keyboard::is_left()
{
	return CL_Keyboard::get_keycode( left_key );
}

bool Controller_Keyboard::is_right()
{
	return CL_Keyboard::get_keycode( right_key );
}

bool Controller_Keyboard::is_up()
{
	return CL_Keyboard::get_keycode( up_key );
}

bool Controller_Keyboard::is_down()
{
	return CL_Keyboard::get_keycode( down_key );
}

bool Controller_Keyboard::is_bomb()
{
	return CL_Keyboard::get_keycode( bomb_key );
}

/* Joystick Controller */

Controller_Joystick::Controller_Joystick(int joystick_nr) : Controller()
{
	assert( joystick_nr < CL_Input::joysticks.get_num_items() );
	joystick = CL_Input::joysticks[joystick_nr];
	
	axes.add( joystick->get_axis(0) );
	axes.add( joystick->get_axis(1) );
	
	buttons.add( joystick->get_button(0) );
}

void Controller_Joystick::update()
{
}

void Controller_Joystick::reset()
{
}

bool Controller_Joystick::is_left()
{
	return axes[0]->get_pos() < -0.2f;
}

bool Controller_Joystick::is_right()
{
	return axes[0]->get_pos() > 0.2f;
}

bool Controller_Joystick::is_up()
{
	return axes[1]->get_pos() < -0.2f;
}

bool Controller_Joystick::is_down()
{
	return axes[1]->get_pos() > 0.2f;
}

bool Controller_Joystick::is_bomb()
{
	return buttons[0]->is_pressed();
}



