/***************************************************************************
                          Controller.h  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Controller.h,v 1.9 1999/05/15 15:56:45 dk Exp $

#ifndef Controller_h
#define Controller_h

#include <ClanLib/clanlib.h>

#include "GameObject.h"
class Bomber;

class Controller
{
public:
	Controller();

	typedef enum
	{
		AI		 = 0,
		KEYMAP_1   = 1,
		KEYMAP_2   = 2,
		KEYMAP_3   = 3,
		JOYSTICK_1 = 4,
		JOYSTICK_2 = 5,
		JOYSTICK_3 = 6,
	} CONTROLLER_TYPE;
	
	static Controller*	create( CONTROLLER_TYPE _type );
	
	void attach(Bomber* _bomber);

	virtual void update() = 0;
	virtual void reset() = 0;
	virtual bool is_left() = 0;
	virtual bool is_right() = 0;
	virtual bool is_up() = 0;
	virtual bool is_down() = 0;
	virtual bool is_bomb() = 0;
	
	Bomber* bomber;
};

class Controller_Keyboard : public Controller
{
public:
	Controller_Keyboard (int keymap_nr);
	virtual ~Controller_Keyboard () {};
	
	void update();
	void reset();
	bool is_left();
	bool is_right();
	bool is_up();
	bool is_down();
	bool is_bomb();
	
private:
	int left_key;
	int right_key;
	int up_key;
	int down_key;
	int bomb_key;
};

class Controller_Joystick : public Controller
{
public:
	Controller_Joystick (int joystick_nr);
	virtual ~Controller_Joystick () {};

	void update();
	void reset();
	bool is_left();
	bool is_right();
	bool is_up();
	bool is_down();
	bool is_bomb();

private:
	CL_InputDevice*	joystick;
	
	CL_List<CL_InputAxis>	axes;
	CL_List<CL_InputButton>	buttons;
};

#endif
















