/***************************************************************************
                          Controller_AI.cpp  -  description                              
                             -------------------                                         
    begin                : Sat Apr 10 1999                                           
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp                         
    email                : clanbomber@fischlustig.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "Controller_AI.h"
#include "Bomber.h"
#include "Bomb.h"
#include "Map.h"
#include "ClanBomber.h"
#include "Timer.h"

/* ---------- Job ----------- */

Job::Job( Controller_AI* _controller )
{
	finished = false;
	obsolete = false;
	controller = _controller;
	bomber = controller->bomber;
	app = bomber->app;
}

Job::~Job()
{
}

bool Job::is_finished()
{
	return finished;
}

bool Job::is_obsolete()
{
	return obsolete;
}

void Job::execute()
{
}

void Job::init()
{
}

/* ---------- Job_Go ----------- */

Job_Go::Job_Go( Controller_AI* _controller, GameObject::Direction _dir, int _distance ) : Job( _controller )
{
	distance = _distance;
	dir = _dir;
	init();
}

Job_Go::~Job_Go()
{
	controller->dir = GameObject::DIR_NONE;
}

void Job_Go::execute()
{
	controller->dir = dir;
	switch(dir)
	{
		case GameObject::DIR_UP:
			if (controller->bomber->get_y() <= start-distance+1)
			{
				finished = true;
				controller->dir = GameObject::DIR_NONE;
			}
		break;
		case GameObject::DIR_DOWN:
			if (controller->bomber->get_y() >= start+distance-1)
			{
				finished = true;
				controller->dir = GameObject::DIR_NONE;
			}
		break;
		case GameObject::DIR_LEFT:
			if (controller->bomber->get_x() <= start-distance+1)
			{
				finished = true;
				controller->dir = GameObject::DIR_NONE;
			}
		case GameObject::DIR_RIGHT:
			if (controller->bomber->get_x() >= start+distance-1)
			{
				finished = true;
				controller->dir = GameObject::DIR_NONE;
			}
		break;
		default:
			obsolete = true;
			controller->dir = GameObject::DIR_NONE;
		break;
	}
	if (controller->bomber->is_stopped())
	{
		obsolete = true;
		controller->dir = GameObject::DIR_NONE;
	}
}

void Job_Go::init()
{
	switch(dir)
	{
		case GameObject::DIR_UP:
		case GameObject::DIR_DOWN:
			start = controller->bomber->get_y();
		break;
		case GameObject::DIR_LEFT:
		case GameObject::DIR_RIGHT:
			start = controller->bomber->get_x();
		break;
		default:
			obsolete = true;
			controller->dir = GameObject::DIR_NONE;
		break;
	}
}

/* ---------- Job_Put_Bomb ----------- */

Job_PutBomb::Job_PutBomb( Controller_AI* _controller ) : Job( _controller )
{
}

Job_PutBomb::~Job_PutBomb()
{
	controller->put_bomb = false;
}

void Job_PutBomb::execute()
{
	controller->put_bomb = true;
	if (controller->bomber->app->map->mapfield[(controller->bomber->get_x()+20)/40][(controller->bomber->get_y()+20)/40]->bomb)
	{
		finished = true;
		return;
	}
	if (controller->bomber->get_cur_bombs() == 0)
	{
		finished = true;
		return;
	}
}

/* ---------- Job_Wait ----------- */

Job_Wait::Job_Wait( Controller_AI* _controller, float _duration ) : Job( _controller )
{
	duration = _duration;
}

Job_Wait::~Job_Wait()
{
}

void Job_Wait::execute()
{
	duration -= Timer::time_elapsed();
	controller->put_bomb = false;
	controller->dir = GameObject::DIR_NONE;
	if (duration <= 0)
	{
		finished = true;
	}
}

Controller_AI::Controller_AI() : Controller()
{
}

void Controller_AI::reset()
{
	dir = GameObject::DIR_NONE;
	put_bomb = false;
	
	for (int i=0; i<jobs.get_num_items(); i++)
	{
		delete jobs[i];
	}
	jobs.empty_list();
}

void Controller_AI::update()
{
	if (job_ready())
	{
		do_job();
	}
}

bool Controller_AI::job_ready()
{
	if (jobs.get_num_items() == 0)
	{
		find_new_jobs();
		return jobs.get_num_items() > 0;
	}
	
	if (jobs[0]->is_finished())
	{
		delete jobs[0];
		jobs.del(jobs.first);
		if (jobs.get_num_items() > 0)
		{
			jobs[0]->init();
			return true;
		}
		return job_ready();
	}

	if (jobs[0]->is_obsolete())
	{
		for (int i=0; i<jobs.get_num_items(); i++)
		{
			delete jobs[i];
		}
		jobs.empty_list();
		return job_ready();
	}
	
	return true;
}

void Controller_AI::find_new_jobs()
{
	if (avoid_bombs())
	{
		return;
	}
	if (fetch_extras())
	{
		return;
	}
/*	if (destroy_boxes())
	{
		return;
	}
	if (kill_enemies())
	{
		return;
	}*/
}

void Controller_AI::do_job()
{
	assert(jobs.get_num_items() > 0);
	jobs[0]->execute();
}

bool Controller_AI::avoid_bombs()
{
	int x = (bomber->get_x()+20)/40;
	int y = (bomber->get_y()+20)/40;
	
	if ( is_hotspot(x,y) )
	{
		return search_save_place( x, y );
	}
	
	return false;
}

bool Controller_AI::search_save_place(int x, int y, GameObject::Direction from)
{
        /* Diese Routine muss komplett neu geschrieben werden */

        if (!is_hotspot(x,y) || jobs.get_num_items() > 8)
        {
                return true;
        }

	if ((from!=GameObject::DIR_RIGHT) && (x<MAP_WIDTH) && (bomber->app->map->mapfield[x+1][y]->type==MapField::GROUND) && (bomber->app->map->mapfield[x+1][y]->bomb==NULL))
	{
		jobs.add( new Job_Go( this, GameObject::DIR_RIGHT, 40 ) );
		if (search_save_place( x+1, y, GameObject::DIR_LEFT ))
		{
			return true;
		} else
		{
			jobs.del(jobs.last);
		}
	}
	
	if ((from!=GameObject::DIR_LEFT) && (x>0) && (bomber->app->map->mapfield[x-1][y]->type==MapField::GROUND) && (bomber->app->map->mapfield[x-1][y]->bomb==NULL))
	{
		jobs.add( new Job_Go( this, GameObject::DIR_LEFT, 40 ) );
		if (search_save_place( x-1, y, GameObject::DIR_RIGHT ))
		{
			return true;
		} else
		{
			jobs.del(jobs.last);
		}
	}
	
	if ((from!=GameObject::DIR_DOWN) && (y<MAP_HEIGHT) && (bomber->app->map->mapfield[x][y+1]->type==MapField::GROUND) && (bomber->app->map->mapfield[x][y+1]->bomb==NULL))
	{
		jobs.add( new Job_Go( this, GameObject::DIR_DOWN, 40 ) );
		if (search_save_place( x, y+1, GameObject::DIR_UP ))
		{
			return true;
		} else
		{
			jobs.del(jobs.last);
		}
	}

	if ((from!=GameObject::DIR_UP) && (y>0) && (bomber->app->map->mapfield[x][y-1]->type==MapField::GROUND) && (bomber->app->map->mapfield[x][y-1]->bomb==NULL))
	{
		jobs.add( new Job_Go( this, GameObject::DIR_UP, 40 ) );
		if (search_save_place( x, y-1, GameObject::DIR_DOWN ))
		{
			return true;
		} else
		{
			jobs.del(jobs.last);
		}
	}
		
	return false;
}

bool Controller_AI::is_hotspot(int x, int y)
{
	Bomb* bomb;
	
	int i = x;
	while (i<MAP_WIDTH && bomber->app->map->mapfield[i][y]->type <= MapField::GROUND)
	{
		bomb = bomber->app->map->mapfield[i][y]->bomb;
		if (bomb  &&  x >= i-bomb->get_power())
		{
			return true;
		}
		i++;
	}

	i = x;
	while (i>0 && bomber->app->map->mapfield[i][y]->type <= MapField::GROUND)
	{
		bomb = bomber->app->map->mapfield[i][y]->bomb;
		if (bomb  &&  x <= i+bomb->get_power())
		{
			return true;
		}
		i--;
	}

	i = y;
	while (i<MAP_HEIGHT && bomber->app->map->mapfield[x][i]->type <= MapField::GROUND)
	{
		bomb = bomber->app->map->mapfield[x][i]->bomb;
		if (bomb  &&  y >= i-bomb->get_power())
		{
			return true;
		}
		i++;
	}
	
	i = y;
	while (i>0 && bomber->app->map->mapfield[x][i]->type <= MapField::GROUND)
	{
		bomb = bomber->app->map->mapfield[x][i]->bomb;
		if (bomb  &&  y <= i+bomb->get_power())
		{
			return true;
		}
		i--;
	}
	return false;
}

bool Controller_AI::fetch_extras()
{
	int x = (bomber->get_x()+20)/40;
	int y = (bomber->get_y()+20)/40;
	GameObject::Direction dirr = GameObject::Direction(rand()%4);
	
	switch (dirr)
	{
		case GameObject::DIR_RIGHT:
			if (is_hotspot(x+1,y) || bomber->app->map->mapfield[x+1][y]->type != MapField::GROUND)
			{
				return false;
			}
		break;
		case GameObject::DIR_LEFT:
			if (is_hotspot(x-1,y) || bomber->app->map->mapfield[x-1][y]->type != MapField::GROUND)
			{
				return false;
			}
		break;
		case GameObject::DIR_DOWN:
			if (is_hotspot(x,y+1) || bomber->app->map->mapfield[x][y+1]->type != MapField::GROUND)
			{
				return false;
			}
		break;
		case GameObject::DIR_UP:
			if (is_hotspot(x,y-1) || bomber->app->map->mapfield[x][y-1]->type != MapField::GROUND)
			{
				return false;
			}
		break;
		default:
		break;
	}
	jobs.add( new Job_Go( this, dirr, 40 ) );
	
	if (rand()%5 == 0)
	{
		jobs.add( new Job_PutBomb(this) );
	}
	return true;
}

bool Controller_AI::is_left()
{
	return dir==GameObject::DIR_LEFT;
}

bool Controller_AI::is_right()
{
	return dir==GameObject::DIR_RIGHT;
}

bool Controller_AI::is_up()
{
	return dir==GameObject::DIR_UP;
}

bool Controller_AI::is_down()
{
	return dir==GameObject::DIR_DOWN;
}

bool Controller_AI::is_bomb()
{
	return put_bomb;
}
