/***************************************************************************
                          Controller_AI.h  -  description                              
                             -------------------                                         
    begin                : Sat Apr 10 1999                                           
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp                         
    email                : clanbomber@fischlustig.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
// $Id: Controller_AI.h,v 1.8 1999/05/14 20:54:43 dk Exp $

#ifndef Controller_AI_h
#define Controller_AI_h

#include <ClanLib/clanlib.h>

#include "Controller.h"
#include "Map.h"

class Controller_AI;

class Job
{
public:
	Job( Controller_AI* _controller );
	virtual ~Job();
	bool is_finished();
	bool is_obsolete();
	virtual void execute();
	virtual void init();
	
protected:
	bool	finished;
	bool	obsolete;
	
	Controller_AI*			controller;
	Bomber*				bomber;
	ClanBomberApplication*	app;
};

class Job_Go : public Job
{
public:
	Job_Go( Controller_AI* _controller, GameObject::Direction _dir, int _distance );
	virtual ~Job_Go();
	void execute();
	void init();
	
protected:
	GameObject::Direction	dir;
	int					distance;
	int					start;
};

class Job_PutBomb : public Job
{
public:
	Job_PutBomb( Controller_AI* _controller );
	virtual ~Job_PutBomb();
	void execute();
};

class Job_Wait : public Job
{
public:
	Job_Wait( Controller_AI* _controller, float _duration );
	virtual ~Job_Wait();
	void execute();
protected:
	float duration;
};

class Controller_AI : public Controller
{
public:
	Controller_AI ();
	virtual ~Controller_AI () {};

	void update();
	void reset();
	bool is_left();
	bool is_right();
	bool is_up();
	bool is_down();
	bool is_bomb();
	
	GameObject::Direction	dir;
	bool					put_bomb;

private:
	CL_List<Job>			jobs;
	
	bool job_ready();
	void do_job();
	void find_new_jobs();
	
	bool avoid_bombs();
	bool fetch_extras();
	bool is_hotspot(int x, int y);
	bool search_save_place(int x, int y, GameObject::Direction from = GameObject::DIR_NONE);
};



#endif

