/***************************************************************************
                          Explosion.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Explosion.cpp,v 1.13 1999/05/15 10:46:03 dk Exp $


#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>


#include "Explosion.h"
#include "ClanBomber.h"
#include "Extra.h"
#include "Timer.h"

Explosion::Explosion( int _x, int _y, int _power, Bomber *_bomber, ClanBomberApplication *_app ) : GameObject( _x, _y, _app )
{
	X = (_x+20) / 40;
	Y = (_y+20) / 40;
	z = Z_EXPLOSION;
	power = _power;
	detonation_period = 0.5f;
	bomber = _bomber;

	find_lengths();

	app->sfx_explode->play();
	app->objects.add( this );
}

Explosion::~Explosion()
{
}

void Explosion::find_lengths()
{
	int i;
	for (i=1; i<=power; i++)
	{
		if ((X-i == 0) || (app->map->mapfield[X-i][Y]->type > MapField::GROUND))
			break;
	}
	l_left = i-1;
	for (i=1; i<=power; i++)
	{
		if ((X+i == MAP_WIDTH-1) || (app->map->mapfield[X+i][Y]->type > MapField::GROUND))
			break;
	}
	l_right = i-1;
	for (i=1; i<=power; i++)
	{
		if ((Y-i == 0) || (app->map->mapfield[X][Y-i]->type > MapField::GROUND))
			break;
	}
	l_up = i-1;
	for (i=1; i<=power; i++)
	{
		if ((Y+i == MAP_HEIGHT-1) || (app->map->mapfield[X][Y+i]->type > MapField::GROUND))
			break;
	}
	l_down = i-1;
}

void Explosion::show()
{
	int anim = 14;
	if (detonation_period < 0.4f)
		anim = 7;
	if (detonation_period < 0.3f)
		anim = 0;
	if (detonation_period < 0.2f)
		anim = 7;
	if (detonation_period < 0.1f)
		anim = 14;

	app->sur_explode->put_screen( x+60, y+40, EXPLODE_X+anim );
	int i;
	for (i=1; i<=l_left; i++)
	{
		if (i == power)
		{
			app->sur_explode->put_screen( x-i*40+60, y+40, EXPLODE_LEFT+anim );
		} else
		{
			app->sur_explode->put_screen( x-i*40+60, y+40, EXPLODE_H+anim );
		}
	}
	for (i=1; i<=l_right; i++)
	{
		if (i == power)
		{
			app->sur_explode->put_screen( x+i*40+60, y+40, EXPLODE_RIGHT+anim );
		} else
		{
			app->sur_explode->put_screen( x+i*40+60, y+40, EXPLODE_H+anim );
		}
	}
	for (i=1; i<=l_up; i++)
	{
		if (i == power)
		{
			app->sur_explode->put_screen( x+60, y-i*40+40, EXPLODE_UP+anim );
		} else
		{
			app->sur_explode->put_screen( x+60, y-i*40+40, EXPLODE_V+anim );
		}
	}
	for (i=1; i<=l_down; i++)
	{
		if (i == power)
		{
			app->sur_explode->put_screen( x+60, y+i*40+40, EXPLODE_DOWN+anim );
		} else
		{
			app->sur_explode->put_screen( x+60, y+i*40+40, EXPLODE_V+anim );
		}
	}
}

void Explosion::kill_bombers()
{
	CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{
		int bx = (bomber_object_counter()->get_x()+20)/40;
		int by = (bomber_object_counter()->get_y()+20)/40;
		
		if ((by==Y && bx>=X-l_left && bx<=X+l_right) || (bx==X && by>=Y-l_up && by<=Y+l_down))
		{
			if (bomber_object_counter()->die() && (bomber_object_counter()!=bomber))
			{
				bomber->inc_kills();
			}
		}
	}
}

void Explosion::destroy_extras()
{
	CL_Iterator<GameObject> object_counter(app->objects);
	while (object_counter.next() != NULL)
	{
		if (object_counter()->get_type() == GameObject::EXTRA)
		{
			int bx = (object_counter()->get_x()+20)/40;
			int by = (object_counter()->get_y()+20)/40;
			
			if ((by==Y && bx>=X-l_left && bx<=X+l_right) || (bx==X && by>=Y-l_up && by<=Y+l_down))
			{
				if (!(object_counter()->is_flying()))
				{	
					((Extra*)object_counter())->destroy();
				}
			}
		}
	}
}

int Explosion::get_power()
{
	return power;
}

void Explosion::act()
{
	do_destruction();
	detonate_other_bombs();
	kill_bombers();
	destroy_extras();

	detonation_period -= Timer::time_elapsed();
	if (detonation_period < 0)
	{
		delete_me = true;
	}
}

void Explosion::do_destruction()
{
	app->map->mapfield[X][Y]->destroy();
	
	if (l_left < power)
	{
		app->map->mapfield[X-l_left-1][Y]->destroy();
	}
	if (l_right < power)
	{
		app->map->mapfield[X+l_right+1][Y]->destroy();
	}
	if (l_up < power)
	{
		app->map->mapfield[X][Y-l_up-1]->destroy();
	}
	if (l_down < power)
	{
		app->map->mapfield[X][Y+l_down+1]->destroy();
	}
}

void Explosion::detonate_other_bombs()
{
	int i;
	for (i=1; i<=l_left; i++)
	{
		if (app->map->mapfield[X-i][Y]->bomb)
		{
			app->map->mapfield[X-i][Y]->bomb->explode_delayed();
			break;
		}
	}
	for (i=1; i<=l_right; i++)
	{
		if (app->map->mapfield[X+i][Y]->bomb)
		{
			app->map->mapfield[X+i][Y]->bomb->explode_delayed();
			break;
		}
	}
	for (i=1; i<=l_up; i++)
	{
		if (app->map->mapfield[X][Y-i]->bomb)
		{
			app->map->mapfield[X][Y-i]->bomb->explode_delayed();
			break;
		}
	}
	for (i=1; i<=l_down; i++)
	{
		if (app->map->mapfield[X][Y+i]->bomb)
		{
			app->map->mapfield[X][Y+i]->bomb->explode_delayed();
			break;
		}
	}
}




