/***************************************************************************
                          GameObject.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: GameObject.cpp,v 1.23 1999/05/15 14:15:41 andi Exp $

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "GameObject.h"
#include "MapField.h"
#include "ClanBomber.h"
#include "Timer.h"

GameObject::GameObject( int _x, int _y, ClanBomberApplication *_app )
{
	app = _app;

	z = 0;
		
	x = _x;
	y = _y;
	orig_x = _x;
	orig_y = _y;
	delete_me = false;
	remainder =0;
	speed = 240;
	
	can_kick = false;
	can_pass_bomber = false;
	flying = false;
	falling = false;
	fly_power = 0;
	fallen_down = false;
	stopped = false;
}

GameObject::~GameObject()
{
}

bool GameObject::move_right()
{
	MapField* right_mapfield = app->map->mapfield[(x+40)/40][(y+20)/40];

	if (right_mapfield->type!=MapField::GROUND && right_mapfield->type!=MapField::NONE )
	{
		return false;
	}
		
	if ((right_mapfield->bomb==NULL) || ((get_type()==BOMB) && (right_mapfield->bomb==this)) || (x%40>19))
	{
		x++;
			
		if (y%40 > 20)
		{
			if (app->map->mapfield[(x+40)/40][(y-20)/40]->type > MapField::GROUND)
			{
				y++;
			}
		} else
		if (y%40 > 0)
		{
			if (app->map->mapfield[(x+40)/40][(y+60)/40]->type > MapField::GROUND)
			{
				y--;
			}
		}

		if (!can_pass_bomber)
		{
	          CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
			while (bomber_object_counter.next() != NULL)
			{
				if (right_mapfield == app->map->mapfield[(bomber_object_counter()->get_x()+20)/40 ][ (bomber_object_counter()->get_y()+20)/40])
				{
					if (bomber_object_counter() != this)
					{
						x--;
						return false;
					}
				}
			}
		}
	}
    	else
	{
		if (can_kick  && (right_mapfield->bomb->get_cur_dir() == DIR_NONE))
		{
			right_mapfield->bomb->kick(DIR_RIGHT);
		}
		return false;
	}
	
	return true;
}

bool GameObject::move_left()
{
	MapField* left_mapfield = app->map->mapfield[(x-1)/40][(y+20)/40];
	if (left_mapfield->type!=MapField::GROUND && left_mapfield->type!=MapField::NONE )
	{
		return false;
	}
		
	if ((left_mapfield->bomb==NULL) || ((get_type()==BOMB) && (left_mapfield->bomb==this)) || (x%40<20))
	{
		x--;
		
		if (y%40 > 20)
		{
			if (app->map->mapfield[(x-1)/40][(y-20)/40]->type > MapField::GROUND)
			{
				y++;
			}
		} else
		if (y%40 > 0)
		{
			if (app->map->mapfield[(x-1)/40][(y+60)/40]->type > MapField::GROUND)
			{
				y--;
			}
		}
			
		if (!can_pass_bomber)
		{
	          CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
			while (bomber_object_counter.next() != NULL)
				{
				if (left_mapfield == app->map->mapfield[(bomber_object_counter()->get_x()+20)/40 ][ (bomber_object_counter()->get_y()+20)/40])
				{			
					if (bomber_object_counter() != this)
					{
						x++;
						return false;
					}
				}
			}
		}
	}
    	else
	{
		if (can_kick  && (left_mapfield->bomb->get_cur_dir() == DIR_NONE))
		{
			left_mapfield->bomb->kick(DIR_LEFT);
		}
		return false;
	}	
	return true;
}

bool GameObject::move_up()
{
	MapField* up_mapfield = app->map->mapfield[(x+20)/40][(y-1)/40];
	if (up_mapfield->type!=MapField::GROUND && up_mapfield->type!=MapField::NONE )
	{
		return false;
	}
		
	if ((up_mapfield->bomb==NULL) || ((get_type()==BOMB) && (up_mapfield->bomb==this)) || (y%40<20))
	{
		y--;
			
		if (x%40 > 20)
		{
			if (app->map->mapfield[(x-20)/40][(y-1)/40]->type > MapField::GROUND)
			{
				x++;
			}
		} else
		if (x%40 > 0)
		{
			if (app->map->mapfield[(x+60)/40][(y-1)/40]->type > MapField::GROUND)
			{
				x--;
			}
		}
			
		if (!can_pass_bomber)
		{
	          CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
			while (bomber_object_counter.next() != NULL)
			{
				if (up_mapfield == app->map->mapfield[(bomber_object_counter()->get_x()+20)/40 ][ (bomber_object_counter()->get_y()+20)/40])
				{			
					if (bomber_object_counter() != this)
					{
						y++;
						return false;
					}
				}
			}
		}
	}
    	else
	{
		if (can_kick  && (up_mapfield->bomb->get_cur_dir() == DIR_NONE))
		{
			up_mapfield->bomb->kick(DIR_UP);
		}
		return false;
	}
	
	return true;
}

bool GameObject::move_down()
{
	MapField* down_mapfield = app->map->mapfield[(x+20)/40][(y+40)/40];
	if (down_mapfield->type!=MapField::GROUND && down_mapfield->type!=MapField::NONE )
	{
		return false;
	}
	
	if ((down_mapfield->bomb==NULL) || ((get_type()==BOMB) && (down_mapfield->bomb==this)) || (y%40>19))
	{
		y++;
			
		if (x%40 > 20)
		{
			if (app->map->mapfield[(x-20)/40][(y+40)/40]->type > MapField::GROUND)
			{
				x++;
			}
		} else
		if (x%40 > 0)
		{
			if (app->map->mapfield[(x+60)/40][(y+40)/40]->type > MapField::GROUND)
			{
				x--;
			}
		}
			
		if (!can_pass_bomber)
		{
	          CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
			while (bomber_object_counter.next() != NULL)
			{
				if (down_mapfield == app->map->mapfield[(bomber_object_counter()->get_x()+20)/40 ][ (bomber_object_counter()->get_y()+20)/40])
				{			
					if (bomber_object_counter() != this)
					{
						y--;
						return false;
					}
				}
			}
		}
	}
    	else
	{
		if (can_kick  && (down_mapfield->bomb->get_cur_dir() == DIR_NONE))
		{
			down_mapfield->bomb->kick(DIR_DOWN);
		}
		return false;
	}
	
	return true;
}

bool GameObject::fly_right()
{
	MapField* right_mapfield = app->map->mapfield[(x+40)/40][(y+20)/40];
	if (right_mapfield->type==MapField::WALL)
	{
		return false;
	}		
	x++;			
	if (y%40 > 20)
	{
		if (app->map->mapfield[(x+40)/40][(y-20)/40]->type != MapField::GROUND)
		{
			y++;
		}
	}else	
	if (y%40 > 0)
	{
		if (app->map->mapfield[(x+40)/40][(y+60)/40]->type != MapField::GROUND)
		{
			y--;
		}
	}
	return true;
}

bool GameObject::fly_left()
{
	MapField* left_mapfield = app->map->mapfield[(x-1)/40][(y+20)/40];
	if (left_mapfield->type==MapField::WALL)
	{
		return false;
	}
		
	x--;
		
	if (y%40 > 20)
	{
		if (app->map->mapfield[(x-1)/40][(y-20)/40]->type != MapField::GROUND)
		{
			y++;
		}
	} else
	if (y%40 > 0)
	{
		if (app->map->mapfield[(x-1)/40][(y+60)/40]->type != MapField::GROUND)
		{
			y--;
		}
	}
			
	return true;
}

bool GameObject::fly_up()
{
	MapField* up_mapfield = app->map->mapfield[(x+20)/40][(y-1)/40];
	if (up_mapfield->type==MapField::WALL)
	{
		return false;
	}
		
	y--;
	
	if (x%40 > 20)
	{
		if (app->map->mapfield[(x-20)/40][(y-1)/40]->type != MapField::GROUND)
		{
			x++;
		}
	} else
	if (x%40 > 0)
	{
		if (app->map->mapfield[(x+60)/40][(y-1)/40]->type != MapField::GROUND)
		{
			x--;
		}
	}			
	return true;
}

bool GameObject::fly_down()
{
	MapField* down_mapfield = app->map->mapfield[(x+20)/40][(y+40)/40];
	if (down_mapfield->type==MapField::WALL)
	{
		return false;
	}
	
	y++;
		
	if (x%40 > 20)
	{
		if (app->map->mapfield[(x-20)/40][(y+40)/40]->type != MapField::GROUND)
		{
			x++;
		}
	} else
	if (x%40 > 0)
	{
		if (app->map->mapfield[(x+60)/40][(y+40)/40]->type != MapField::GROUND)
		{
			x--;
		}
	}
	return true;
}

bool GameObject::move()
{
	if (flying)
	{
		return fly();
	}
	else
	{
		int span = (int)(Timer::time_elapsed()*speed);
	
		remainder += Timer::time_elapsed()*speed - (int)(Timer::time_elapsed()*speed);
		span += (int)(remainder);
		remainder -= (int)(remainder);
	
		for (int i=0; i<span; i++)
		{
			switch (cur_dir)
			{
				case DIR_LEFT:
					if (!move_left())
					{
						stop();
						return false;
					}
				break;
				case DIR_RIGHT:
					if (!move_right())
					{
						stop();
						return false;
					}
				break;
				case DIR_UP:
					if (!move_up())
					{
						stop();
						return false;
					}
				break;
				case DIR_DOWN:
					if (!move_down())
					{
						stop();
						return false;
					}
				break;
				default:
				break;
			}
		}
	}
	return true;
}

bool GameObject::fly ()
{
	int span = (int)(Timer::time_elapsed()*speed);
	
	remainder += Timer::time_elapsed()*speed - (int)(Timer::time_elapsed()*speed);
	span += (int)(remainder);
	remainder -= (int)(remainder);
	
	for (int i=0; i<span; i++)
	{
		switch (cur_dir)
		{
			case DIR_LEFT:
				if (!fly_left())
				{
					stop();
					return false;
				}
			break;
			case DIR_RIGHT:
				if (!fly_right())
				{
					stop();
					return false;
				}
			break;
			case DIR_UP:
				if (!fly_up())
				{
					stop();
					return false;
				}
			break;
			case DIR_DOWN:
				if (!fly_down())
				{
					stop();
					return false;
				}
			break;
			default:
			break;
		}
	}
	return true;
}


void GameObject::act()
{
	stopped = false;
	if (flying)
	{
		continue_flying();
	}
	if (falling)
	{
		continue_falling();
	}
}
void GameObject::fly_to (int _x, int _y)
{
	if (!flying)
	{	
		destination_x = _x;
		destination_y = _y;
		source_x = x;
		source_y = y;
		start_fly_power = sqrt ((_x - x ) * (_x - x ) + ( _y - y) * ( _y -y ));
		fly_power = start_fly_power;
		flying = true;
		z = Z_FLYING_BOMB;
	}
}

void GameObject::continue_flying()
{
	fly_power-=Timer::time_elapsed()*speed;
	if (fly_power < 0)
	{
		flying=false;
		fly_power=0;
		z = Z_EXTRA;
	}
	x = source_x +  (int)(((start_fly_power - fly_power)/start_fly_power) * (destination_x - source_x));
	y = source_y +  (int)(((start_fly_power - fly_power)/start_fly_power) * (destination_y - source_y));

}



bool GameObject::move( int _speed, Direction _dir )
{
	if (_dir==DIR_NONE && cur_dir!=DIR_NONE)
	{
		return move( _speed, cur_dir );
	}

	int span = (int)(Timer::time_elapsed() * _speed);
	
	remainder += Timer::time_elapsed() * _speed - (int)(Timer::time_elapsed() * _speed);
	span += (int)(remainder);
	remainder -= (int)(remainder);
	
	for (int i=0; i<span; i++)
	{
		switch (_dir)
		{
			case DIR_LEFT:
				if (!move_left())
				{
					stop();
					return false;
				}
			break;
			case DIR_RIGHT:
				if (!move_right())
				{
					stop();
					return false;
				}
			break;
			case DIR_UP:
				if (!move_up())
				{
					stop();
					return false;
				}
			break;
			case DIR_DOWN:
				if (!move_down())
				{
					stop();
					return false;
				}
			break;
			default:
			break;
		}
	}
	return true;
}

void GameObject::fall()
{
	if (!falling && !flying)
	{	
		falling = true;
		z = Z_FALLING;
//		speed = 40;
		fall_countdown = 1.0f;
	}
}
void GameObject::continue_falling()
{
	speed=fall_countdown*60;
	fall_countdown -= Timer::time_elapsed();
	if (fall_countdown < 0)
	{
		fallen_down = true;
		falling = false;
	}
}

void GameObject::stop()
{
	stopped = true;
}

void GameObject::snap()
{
	x = ((x+20)/40) *40;
	y = ((y+20)/40) *40;
}

void GameObject::set_dir ( Direction _dir)
{
	cur_dir = _dir;
}
void GameObject::set_pos( int _x, int _y )
{
	x = _x;
	y = _y;
}

void GameObject::set_orig( int _x, int _y )
{
	orig_x = _x;
	orig_y = _y;
}

void GameObject::move_pos( int _x, int _y )
{
	x += _x;
	y += _y;
}

int GameObject::get_x()
{
	return x;
}

int GameObject::get_y()
{
	return y;
}

int GameObject::get_z()
{
	return z;
}

bool GameObject::is_flying()
{
	return flying;
}

bool GameObject::is_stopped()
{
	return stopped;
}

GameObject::Direction GameObject::get_cur_dir()
{
	return cur_dir;
}

int GameObject::whats_left()
{
	return app->map->mapfield[(x-1)/40][(y+20)/40]->type;
}

int GameObject::whats_right()
{
	return app->map->mapfield[(x+40)/40][(y+20)/40]->type;
}

int GameObject::whats_up()
{
	return app->map->mapfield[(x+20)/40][(y-1)/40]->type;
}

int GameObject::whats_down()
{
	return app->map->mapfield[(x+20)/40][(y+40)/40]->type;
}
















































