/***************************************************************************
                          GameObject.h  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: GameObject.h,v 1.14 1999/04/12 16:31:20 dk Exp $

#ifndef GameObject_h
#define GameObject_h

class ClanBomberApplication;

#define Z_FLYING_BOMB	6000
#define Z_OBSERVER		5000
#define Z_BOMBER		4000
#define Z_BOMB			3000
#define Z_EXPLOSION		2000
#define Z_EXTRA		1000
#define Z_CORPSE		100
#define Z_GROUND		0
#define Z_FALLING		-1

class GameObject
{
public:
	typedef enum
	{
	  DIR_NONE = -1,
	  DIR_DOWN  = 0,
	  DIR_LEFT  = 1,
	  DIR_UP    = 2,
	  DIR_RIGHT = 3
	} Direction;

	GameObject( int _x, int _y, ClanBomberApplication *_app );
	virtual ~GameObject();

	void set_pos( int _x, int _y );
	void set_dir (Direction _dir);
	void set_orig( int _x, int _y );
	void move_pos( int _x, int _y );
	void snap();
	void fall();
	bool move(int _speed, Direction _dir=DIR_NONE);

	int get_x();
	int get_y();
	int get_z();
	
	bool is_flying();
	bool is_stopped();

	Direction get_cur_dir();

	typedef enum
	{
		BOMB,
		BOMBER,
		BOMBER_KI,
		BOMBER_CORPSE,
		EXPLOSION,
		EXTRA,
		DEBUG,
		OBSERVER
	} ObjectType;

	virtual ObjectType	get_type() = 0;
	virtual void		stop();
	virtual void		show() = 0;
	virtual void		act();
	void fly_to (int _x, int _y);
	void gain_kick();
	void loose_kick();
	
	bool delete_me;
	ClanBomberApplication *app;

protected:
	
	bool move();
	bool move_left();
	bool move_right();
	bool move_up();
	bool move_down();

	bool fly();
	bool fly_left();
	bool fly_right();
	bool fly_up();
	bool fly_down();
	
	int whats_left();
	int whats_right();
	int whats_up();
	int whats_down();

	void continue_falling();


	void continue_flying();
	bool flying;
	bool falling;
	bool fallen_down;
	bool can_kick;
	bool can_pass_bomber;
	int x;
	int y;
	int z;
	int orig_x;
	int orig_y;
	int destination_x;
	int destination_y;
	int source_x;
	int source_y;
	float remainder;
	Direction cur_dir;
	float speed;
	float fall_countdown;
	float fly_power;
	float start_fly_power;

	bool stopped;
};

#endif






































