/***************************************************************************
                          Map.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Map.cpp,v 1.8 1999/04/13 22:54:14 dk Exp $

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "ClanBomber.h"
#include "Map.h"
#include "Config.h"


Map::Map( ClanBomberApplication* _app )
{
	app = _app;

	for (int y=0; y<MAP_HEIGHT; y++)
	{
		for (int x=0; x<MAP_WIDTH; x++)
		{
			mapfield[x][y] = NULL;
		}
	}
}

Map::~Map()
{
}

CL_Vector Map::get_bomber_pos(int nr)
{
	return bomber_pos[nr%8];
}

void Map::load( CL_String name )
{
	clear();
	
	CL_InputSourceProvider *df_clanbomber = CL_InputSourceProvider::create_datafile_provider("clanbomber.dat");

	CL_InputSource *input = df_clanbomber->open_source(name);
	for (int y=0; y<MAP_HEIGHT; y++)
	{
		for (int x=0; x<MAP_WIDTH; x++)
		{
			int cur_tile = input->read_char8();
			switch (cur_tile)
			{
				case 48:
				case 49:
				case 50:
				case 51:
				case 52:
				case 53:
				case 54:
				case 55:
				case 32:
					mapfield[x][y] = new MapField(40*x, 40*y, MapField::GROUND, Config::get_theme(), app);
					break;
				case 42:
					mapfield[x][y] = new MapField(40*x, 40*y, MapField::WALL, Config::get_theme(), app);
					break;
				case 43:
					mapfield[x][y] = new MapField(40*x, 40*y, MapField::D_WALL, Config::get_theme(), app);
					break;
				case 82:
					if ( rand() %3)
					{
						mapfield[x][y] = new MapField(40*x, 40*y, MapField::D_WALL, Config::get_theme(), app);
					}
					else
					{
						mapfield[x][y] = new MapField(40*x, 40*y, MapField::GROUND, Config::get_theme(), app);
					}					
					break;
				default:
					mapfield[x][y] = new MapField(40*x, 40*y, MapField::NONE, Config::get_theme(), app);
					break;
			}
			
			switch (cur_tile)
			{
				case 48:
					bomber_pos[0].x = x;
					bomber_pos[0].y = y;
					break;
				case 49:
					bomber_pos[1].x = x;
					bomber_pos[1].y = y;
					break;
				case 50:
					bomber_pos[2].x = x;
					bomber_pos[2].y = y;
					break;
				case 51:
					bomber_pos[3].x = x;
					bomber_pos[3].y = y;
					break;
				case 52:
					bomber_pos[4].x = x;
					bomber_pos[4].y = y;
					break;
				case 53:
					bomber_pos[5].x = x;
					bomber_pos[5].y = y;
					break;
				case 54:
					bomber_pos[6].x = x;
					bomber_pos[6].y = y;
					break;
				case 55:
					bomber_pos[7].x = x;
					bomber_pos[7].y = y;
					break;
			}
		}
		if (input->read_char8() == 13)
		{
			input->read_char8();
		}
	}

	delete input;
	delete df_clanbomber;
}

void Map::show()
{
	for (int y=0; y<MAP_HEIGHT; y++)
	{
		for (int x=0; x<MAP_WIDTH; x++)
		{
			if (mapfield[x][y]->type!=MapField::NONE)
			{
				if (is_shaken > 0)
				{
					if (mapfield[x][y])
					{
						mapfield[x][y]->show_shaken();
					}
				}
				else
				{
					if (mapfield[x][y])
					{
						mapfield[x][y]->show();
					}
				}
			}
		}
	}
	if (is_shaken > 0)	
	{
		is_shaken--;
	}
}

void Map::refresh_holes()
{
	for (int y=0; y<MAP_HEIGHT; y++)
	{
		for (int x=0; x<MAP_WIDTH; x++)
		{
			if (mapfield[x][y]->type==MapField::NONE)
			{
				if (is_shaken > 0)
				{
					if (mapfield[x][y])
					{
						mapfield[x][y]->show_shaken();
					}
				}
				else
				{
					if (mapfield[x][y])
					{
						mapfield[x][y]->show();
					}
				}
			}
		}
	}
}

void Map::act()
{
	for (int y=0; y<MAP_HEIGHT; y++)
	{
		for (int x=0; x<MAP_WIDTH; x++)
		{
			mapfield[x][y]->act();
		}
	}
}

void Map::shake( int _shake )
{
	is_shaken = _shake;
}


void Map::clear()
{
	for (int y=0; y<MAP_HEIGHT; y++)
	{
		for (int x=0; x<MAP_WIDTH; x++)
		{
			if (mapfield[x][y])
			{
				delete mapfield[x][y];
				mapfield[x][y] = NULL;
			}
		}
	}
	for (int i=0; i<8; i++)
	{
		bomber_pos[i].x = MAP_WIDTH/2;
		bomber_pos[i].y = MAP_HEIGHT/2;
	}
	is_shaken = 0;
}

bool Map::add_bomb(int bx, int by, int bpower, Bomber *bomber)
{
	if ((bx<0) || (by<0) || (bx>=MAP_WIDTH) || (by>=MAP_HEIGHT))
	{
		return false;
	}
	
	if (mapfield[bx][by] == NULL)
	{
		return false;
	}

	if (mapfield[bx][by]->bomb == NULL)
	{
		mapfield[bx][by]->bomb = new Bomb( bx*40, by*40,bpower, bomber, app );
		return true;
	}

	return false;
}











