/***************************************************************************
                          MapField.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: MapField.cpp,v 1.12 1999/04/06 18:41:20 dk Exp $

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "MapField.h"
#include "ClanBomber.h"
#include "Config.h"
#include "Timer.h"

#include "Extra_Bomb.h"
#include "Extra_Power.h"
#include "Extra_Skateboard.h"
#include "Extra_Kick.h"
#include "Extra_Glove.h"


MapField::MapField( int _x, int _y, int _type, int _theme, ClanBomberApplication *_app )
{
	x = _x;
	y = _y;
	app = _app;
	type = _type;
	theme = _theme;
	bomb = NULL;
	vanish_countdown = 1.0f;
	vanishing = false;
	destroyed = false;
}

MapField::~MapField()
{
}

void MapField::show()
{
	if (type != NONE)
	{
		if (vanishing)
		{
			CL_Display::fill_rect( x+60, y+40,  x+100, y+80, 0, 0, 0, 1);
			app->sur_maptiles->put_screen( x+80-vanish_countdown*20, y+60-vanish_countdown*20, vanish_countdown, vanish_countdown, type + theme*4 );
		}
		else
		{
			app->sur_maptiles->put_screen( x+60, y+40, type + theme*4 );						
		}
	}
	else
	{
		CL_Display::fill_rect( x+60, y+40,  x+100, y+80, 0, 0, 0, 1);
	}
}
void MapField::show_shaken()
{
	if (type != NONE)
	{		
		if (vanishing)
		{
			CL_Display::fill_rect( x+60, y+40,  x+100, y+80, 0, 0, 0, 1);
			app->sur_maptiles->put_screen( x+80-vanish_countdown*20, y+60-vanish_countdown*20, vanish_countdown, vanish_countdown, type + theme*4 );
		}
		else
		{
			app->sur_maptiles->put_screen( x+60+rand()%3-1, y+40+rand()%3-1, type + theme*4 );
		}
	}
	else
	{
		int xoffset = rand()%3 -1;
		int yoffset = rand()%3 -1;
		CL_Display::fill_rect( x+60+xoffset, y+40+yoffset,  x+100+xoffset, y+80+yoffset, 0, 0, 0, 1);
	}
}

void MapField::spawn_extra()
{
	switch (rand()%8)
	{
		case 0:
			new Extra_Power( x, y, app );
		break;
		case 1:
			new Extra_Bomb( x, y, app );
		break;
		case 2:
			new Extra_Skateboard( x, y, app );
		break;
		case 3:
			switch (rand()%2)
			{
				case 0:
					if (!Config::get_start_kick())
					{
						new Extra_Kick( x, y, app );
					}
				break;
				case 1:
					if (!Config::get_start_glove())
					{
						new Extra_Glove( x, y, app );
					}
				break;
			}
		break;
	}
}

void MapField::destroy()
{
	if (type==D_WALL && !destroyed && !vanishing)
	{
		destroy_countdown = 0.5f;
		destroyed = true;
		type = L_WALL;
	}
}

void MapField::act()
{
	if (destroyed)
	{
		destroy_countdown -= Timer::time_elapsed();
		if (destroy_countdown < 0)
		{
			type = GROUND;
			spawn_extra();
			destroyed = false;
		}
	}
	
	if (vanishing)
	{
		vanish_countdown -= Timer::time_elapsed();
		if (vanish_countdown < 0)
		{
			vanishing = false;
			type = NONE;			
		}
	}
	if (type == MapField::NONE)
	{	
		bomb = NULL;
		CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
		while (bomber_object_counter.next() != NULL)
		{	
			if ((((bomber_object_counter()->get_x()+20)/40 == (x+20)/40)) && (((bomber_object_counter()->get_y()+20)/40 == (y+20)/40)))
			{
				bomber_object_counter()->fall();
			}
		}	
		CL_Iterator<GameObject> object_counter(app->objects);
		while (object_counter.next() != NULL)
		{	
			if ((((object_counter()->get_x()+20)/40 == (x+20)/40)) && (((object_counter()->get_y()+20)/40 == (y+20)/40)))
			{
				object_counter()->fall();
			}
		}	
	}
}

void MapField::vanish()
{
	if (type!=NONE)
	{
		vanishing = true;
	}
}





