/***************************************************************************
                          Menu.cpp  -  description
                             -------------------
    begin                : ?
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp
    email                : clanbomber@fischlustig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: Menu.cpp,v 1.20 1999/04/20 21:53:50 andi Exp $

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "Menu.h"
#include "ClanBomber.h"
#include "Timer.h"

MenuItem::MenuItem( const CL_String _text, int _id, int _parent=0 )
{
	text = _text;
	id = _id;
	parent = _parent;
}

bool MenuItem::has_children()
{
	return children.get_num_items() > 0;
}

void MenuItem::add_child( MenuItem* child )
{
	children.add( child );
}

int MenuItem::get_id() const
{
	return id;
}

int MenuItem::get_parent() const
{
	return parent;
}

CL_String MenuItem::get_text()
{
	return text;
}

void MenuItem::set_text(const CL_String _text)
{
	text = _text;
}


MenuItem_Value::MenuItem_Value( const CL_String _text, int _id, int _parent, int _min, int _max, int _value ) : MenuItem( _text, _id, _parent )
{
	value = _value;
	min = _min;
	max = _max;
	test_value();
}

int MenuItem_Value::get_value()
{
	return value;
}

int MenuItem_Value::get_min()
{
	return min;
}

int MenuItem_Value::get_max()
{
	return max;
}

void MenuItem_Value::set_min( int _min )
{
	min = _min;
	test_value();
}

void MenuItem_Value::set_max( int _max )
{
	max = _max;
	test_value();
}

void MenuItem_Value::set_value( int _value )
{
	value = _value;
	test_value();
}

void MenuItem_Value::inc_value()
{
	value++;
	test_value();
}

void MenuItem_Value::dec_value()
{
	value--;
	test_value();
}

void MenuItem_Value::test_value()
{
	if (value > max)
	{
	  	value = max;
	}
	if (value < min)
	{
		value = min;
	}
}

// *********************
MenuItem_String::MenuItem_String( const CL_String _text, int _id, int _parent, CL_String _string) : MenuItem( _text, _id, _parent )
{
	string = _string;
}

CL_String& MenuItem_String::get_string()
{
	return string;
}

void MenuItem_String::set_string( CL_String  _string )
{
	string = _string;
}

// *********************
MenuItem_StringList::MenuItem_StringList( const CL_String _text, int _id, int _parent, CL_Array<CL_String> _string_list, int _value ) : MenuItem( _text, _id, _parent )
{
	set_strings( _string_list );
	value = _value;
	test_value();
}

CL_String MenuItem_StringList::get_string()
{
	return *(string_list[value]);
}

CL_Array<CL_String> MenuItem_StringList::get_strings()
{
	return string_list;
}

void MenuItem_StringList::set_value( int _value )
{
	value = _value;
	test_value();
}

int MenuItem_StringList::get_value()
{
	return value;
}

void MenuItem_StringList::set_strings( CL_Array<CL_String>  _string_list )
{
	min = 0;
	max = _string_list.get_num_items() - 1;
	for (int i=min; i<=max; i++)
		string_list.add( new CL_String( *(_string_list[i]) ) );
}

void MenuItem_StringList::inc_value()
{
	value++;
	test_value();
}

void MenuItem_StringList::dec_value()
{
	value--;
	test_value();
}

void MenuItem_StringList::test_value()
{
	if (value > max)
	{
	  	value = max;
	}
	if (value < min)
	{
		value = min;
	}
}

// ************************
Menu::Menu( const CL_String& name, ClanBomberApplication* _app )
{
	app = _app;
	current_run_id = 0;
	current_selection = 1;

	key_down = false;
	key_up = false;
	key_enter = false;
	key_esc = false;

	enter = false;

	if (rand()%2)
	{
		sur_titlescreen = CL_Res_Surface::load("App/sur_title_andi", *(app->res_app));
	}
	else
	{
		sur_titlescreen = CL_Res_Surface::load("App/sur_title_dok", *(app->res_app));
	}
	
	key_buffer = CL_Input::keyboards[0]->get_buffer(0);

	items.add( new MenuItem(name, 0, -1) );
}

Menu::~Menu()
{
}

int Menu::execute()
{
	enter = false;
	key_esc = true;
	bool must_redraw = true;
	
	while (1)
	{
		MenuItem* current = get_item_by_id(current_run_id);

		if (must_redraw)
		{
			sur_titlescreen->put_screen(0, 0, 0 );
			CL_Display::fill_rect( 200,50, 600,550, 0,0,0,0.5f);

			int vert = 260 - (current->children.num_items*40) / 2;
			app->fnt_clansoft->print_center( 400, vert, current->get_text() );
			vert += 80;

			int act_draw = 0;
			CL_Iterator<MenuItem> item_counter( current->children );
			while (item_counter.next() != NULL)
			{
				act_draw++;
				if (act_draw == current_selection)
				{
					app->fnt_clansoft->print_left( 235, vert-4, CL_String(">") );
				}
				if (item_counter()->get_type() == MenuItem::MT_VALUE)
				{
					if ((((MenuItem_Value*)item_counter())->get_min() == 0) && (((MenuItem_Value*)item_counter())->get_max() == 1))
					{
						app->fnt_clansoft->print_right( 560, vert, CL_String( ((MenuItem_Value*)item_counter())->get_value() ? "Yes" : "No" ) );				
					}
					else
					{
						app->fnt_clansoft->print_right( 560, vert, CL_String( ((MenuItem_Value*)item_counter())->get_value() ) );
					}
				}
				if (item_counter()->get_type()==MenuItem::MT_STRING)
				{
					app->fnt_clansoft->print_right( 560, vert, ((MenuItem_String*)item_counter())->get_string() );
				}
				if (item_counter()->get_type()==MenuItem::MT_STRINGLIST)
				{
					app->fnt_clansoft->print_right( 560, vert, ((MenuItem_StringList*)item_counter())->get_string() );
				}
				app->fnt_clansoft->print_left( 255, vert, item_counter()->get_text() );
				vert += 40;
			}
			CL_Display::flip_display();
			must_redraw = false;
		}
		
		CL_System::keep_alive();

		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_DOWN) && !key_down)
		{
			current_selection++;
			if (current_selection > current->children.num_items) current_selection = get_item_by_id(current_run_id)->children.num_items;
			must_redraw = true;
		}
		key_down = CL_Keyboard::get_keycode(CL_Keyboard::KEY_DOWN);
	
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_UP) && !key_up)
		{
			current_selection--;
			if (current_selection < 1) current_selection = 1;
			must_redraw = true;
		}
		key_up =CL_Keyboard::get_keycode(CL_Keyboard::KEY_UP);
	
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RIGHT) && !key_right)
		{
			if (current->children.get_item(current_selection-1)->get_type() == MenuItem::MT_VALUE)
			{
				((MenuItem_Value*)(current->children.get_item(current_selection-1)))->inc_value();
			}
			if (current->children.get_item(current_selection-1)->get_type() == MenuItem::MT_STRINGLIST)
			{
				((MenuItem_StringList*)(current->children.get_item(current_selection-1)))->inc_value();
			}
		}
		key_right = CL_Keyboard::get_keycode(CL_Keyboard::KEY_RIGHT);
		
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LEFT) && !key_left)
		{
			if (current->children.get_item(current_selection-1)->get_type() == MenuItem::MT_VALUE)
			{
				((MenuItem_Value*)(current->children.get_item(current_selection-1)))->dec_value();
			}
			if (current->children.get_item(current_selection-1)->get_type() == MenuItem::MT_STRINGLIST)
			{
				((MenuItem_StringList*)(current->children.get_item(current_selection-1)))->dec_value();
			}
		}
		key_left = CL_Keyboard::get_keycode(CL_Keyboard::KEY_LEFT);
	
		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ENTER) && !key_enter)
		{
			if (current->children.get_item(current_selection-1)->has_children())
			{
				current_run_id = current->children.get_item(current_selection-1)->get_id();
				current_selection = 1;
				must_redraw = true;
			} else
			{
				enter = true;
				if (current->children.get_item(current_selection-1)->get_type() == MenuItem::MT_STRING)
				{
					enter_string( (MenuItem_String*)(current->children.get_item(current_selection-1)) );
				}
			}
		}
		key_enter = CL_Keyboard::get_keycode(CL_Keyboard::KEY_ENTER);

		if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE) && !key_esc)
		{
			if (current->get_id() != 0)
			{
				current_run_id = current->get_parent();
				current_selection = 1;
				must_redraw = true;
			}
		}
		key_esc = CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE);

		if (key_left || key_right || enter)
		{
			enter = false;
			return current->children.get_item(current_selection-1)->get_id();
		}
	}
	return -1;			
}

void Menu::scroll_in()
{
	scroll( -290, 260, 600 );
}

void Menu::scroll_out()
{
	scroll( 260, 810, 600 );
}

void Menu::scroll(int _start, int _stop, int _speed)
{
	MenuItem* current = get_item_by_id(current_run_id);
	
	Timer timer;
	
	float i = _start;
	while (i<_stop)
	{
		sur_titlescreen->put_screen(0, 0, 0 );
		CL_Display::fill_rect( 200,i-210, 600,i+290, 0,0,0,0.5f);

		int vert = (int)i - (current->children.num_items*40) / 2;
		app->fnt_clansoft->print_center( 400, vert, current->get_text() );
		vert += 80;

		int act_draw = 0;
		CL_Iterator<MenuItem> item_counter( current->children );
		while (item_counter.next() != NULL)
		{
			act_draw++;
			if (act_draw == current_selection)
			{
				app->fnt_clansoft->print_left( 235, vert-4, CL_String(">") );
			}
			if (item_counter()->get_type() == MenuItem::MT_VALUE)
			{
				if ((((MenuItem_Value*)item_counter())->get_min() == 0) && (((MenuItem_Value*)item_counter())->get_max() == 1))
				{
					app->fnt_clansoft->print_right( 560, vert, CL_String( ((MenuItem_Value*)item_counter())->get_value() ? "Yes" : "No" ) );				
				}
				else
				{
					app->fnt_clansoft->print_right( 560, vert, CL_String( ((MenuItem_Value*)item_counter())->get_value() ) );
				}
			}
			if (item_counter()->get_type()==MenuItem::MT_STRING)
			{
				app->fnt_clansoft->print_right( 560, vert, ((MenuItem_String*)item_counter())->get_string() );
			}
			if (item_counter()->get_type()==MenuItem::MT_STRINGLIST)
			{
				app->fnt_clansoft->print_right( 560, vert, ((MenuItem_StringList*)item_counter())->get_string() );
			}
			app->fnt_clansoft->print_left( 255, vert, item_counter()->get_text() );
			vert += 40;
		}
		
		CL_Display::flip_display();
		CL_System::keep_alive();
		
		i += _speed * timer.time_elapsed(true);
	}
}

void Menu::enter_string(MenuItem_String* item)
{
	while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ENTER))
	{
		CL_System::keep_alive();
	}
	
	key_buffer->clear();
	CL_String new_string = item->get_string();
	
	while (1)
	{
		sur_titlescreen->put_screen(0, 0, 0 );
		CL_Display::fill_rect( 200,300, 600,400, 0,0,0,0.5f);

		app->fnt_clansoft->print_left( 230, 330, item->get_text() );
		
		app->fnt_clansoft->print_left( 380, 330, new_string );
		
		CL_Display::flip_display();
		CL_System::keep_alive();
		
		switch (key_buffer->get_key())
		{
			case CL_Keyboard::KEY_ENTER:
				item->set_string( new_string );
				return;
			break;
			case CL_Keyboard::KEY_BACKSPACE:
				new_string = new_string.mid( 0, new_string.get_length()-1 );
			break;
			case CL_Keyboard::KEY_SPACE:
				new_string += " ";
			break;
			case CL_Keyboard::KEY_A:
				new_string += "a";
			break;
			case CL_Keyboard::KEY_B:
				new_string += "b";
			break;
			case CL_Keyboard::KEY_C:
				new_string += "c";
			break;
			case CL_Keyboard::KEY_D:
				new_string += "d";
			break;
			case CL_Keyboard::KEY_E:
				new_string += "e";
			break;
			case CL_Keyboard::KEY_F:
				new_string += "f";
			break;
			case CL_Keyboard::KEY_G:
				new_string += "g";
			break;
			case CL_Keyboard::KEY_H:
				new_string += "h";
			break;
			case CL_Keyboard::KEY_I:
				new_string += "i";
			break;
			case CL_Keyboard::KEY_J:
				new_string += "j";
			break;
			case CL_Keyboard::KEY_K:
				new_string += "k";
			break;
			case CL_Keyboard::KEY_L:
				new_string += "l";
			break;
			case CL_Keyboard::KEY_M:
				new_string += "m";
			break;
			case CL_Keyboard::KEY_N:
				new_string += "n";
			break;
			case CL_Keyboard::KEY_O:
				new_string += "o";
			break;
			case CL_Keyboard::KEY_P:
				new_string += "p";
			break;
			case CL_Keyboard::KEY_Q:
				new_string += "q";
			break;
			case CL_Keyboard::KEY_R:
				new_string += "r";
			break;
			case CL_Keyboard::KEY_S:
				new_string += "s";
			break;
			case CL_Keyboard::KEY_T:
				new_string += "t";
			break;
			case CL_Keyboard::KEY_U:
				new_string += "u";
			break;
			case CL_Keyboard::KEY_V:
				new_string += "v";
			break;
			case CL_Keyboard::KEY_W:
				new_string += "w";
			break;
			case CL_Keyboard::KEY_X:
				new_string += "x";
			break;
			case CL_Keyboard::KEY_Y:
				new_string += "y";
			break;
			case CL_Keyboard::KEY_Z:
				new_string += "z";
			break;
		}
		if (new_string.get_length() == 1)
		{
			new_string.to_upper();
		}
	}
}

void Menu::add_item( const CL_String& text, int id, int parent )
{
	MenuItem* new_item = new MenuItem( text, id, parent );

	get_item_by_id(parent)->add_child(new_item);
	items.add( new_item );
}

void Menu::add_value( const CL_String& text, int id, int parent, int min, int max, int value )
{
	MenuItem_Value* new_item = new MenuItem_Value( text, id, parent, min, max, value );

	get_item_by_id(parent)->add_child(new_item);
	items.add( new_item );
}

void Menu::add_string( const CL_String& text, int id, int parent, CL_String string )
{
	MenuItem_String*  new_item = new MenuItem_String( text, id, parent, string);
	
	get_item_by_id(parent)->add_child(new_item);
	items.add( new_item );
}

void Menu::add_stringlist( const CL_String& text, int id, int parent, CL_Array<CL_String> string_list, int cur_string )
{
	MenuItem_StringList*  new_item = new MenuItem_StringList( text, id, parent, string_list, cur_string );
	
	get_item_by_id(parent)->add_child(new_item);
	items.add( new_item );
}

MenuItem* Menu::get_item_by_id( int id )
{
	CL_Iterator<MenuItem> item_counter(items);
	while (item_counter.next() != NULL)
	{
		if (item_counter()->get_id() == id)
		{
			return item_counter();
		}
	}
	return NULL;
}
