/***************************************************************************
                          Observer_Classic.cpp  -  description                              
                             -------------------                                         
    begin                : Fri May 14 1999                                           
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp                         
    email                : clanbomber@fischlustig.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "Observer_Classic.h"

#include "ClanBomber.h"
#include "Config.h"
#include "Timer.h"

Observer_Classic::Observer_Classic( int _x, int _y, ClanBomberApplication *_app ) : Observer(_x, _y, _app)
{
	mode = NORMAL;
	game_status = new GameStatus (_app);
	rounds = 0;
	reset_round_time();
	destroy_countdown = 1.0f;
}

Observer_Classic::~Observer_Classic()
{
	delete game_status;
}

void Observer_Classic::act()
{
	round_time -= Timer::time_elapsed();
	if (round_time < 0)
	{
		destroy_countdown-= Timer::time_elapsed();
		if (destroy_countdown <0)
		{
			destroy_countdown+=1.0f;
			int xmapfield = rand() % 17;
			int ymapfield = rand() % 13;
			while (app->map->mapfield[xmapfield][ymapfield]->type!=MapField::GROUND)
			{
				xmapfield = rand() % 17;
				ymapfield = rand() % 13;
			}
			app->map->mapfield[xmapfield][ymapfield]->vanish();
		}
	}
	switch( mode )
	{
		case NORMAL:
			act_normal();
		break;
		case END_ROUND:
			act_waiting_for_explosions();
		break;
	}
}

void Observer_Classic::act_normal()
{
	int c = 0;

	CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{
		if (!bomber_object_counter()->is_dead())
		{
			c++;
		}
		else
		{
			corpse_container.add (bomber_object_counter());
			bomber_object_counter.remove();
		}
	}
	if (c < 2)
	{
		mode = END_ROUND;
	}
}

void Observer_Classic::act_waiting_for_explosions()
{
	CL_Iterator<Bomber> bomber_object_counter_x(app->bomber_objects);
	while (bomber_object_counter_x.next() != NULL)
	{
		if (bomber_object_counter_x()->is_dead())
		{
			corpse_container.add (bomber_object_counter_x());
			bomber_object_counter_x.remove();
		}
	}
	
	CL_Iterator<GameObject> object_counter(app->objects);
	while (object_counter.next() != NULL)
	{
		if (object_counter()->get_type() == GameObject::EXPLOSION)
		{
			return;
		}
		if (object_counter()->get_type() == GameObject::EXTRA && object_counter()->is_flying())
		{
			return;
		}
	}
	
	Bomber* last_bomber = NULL;

	CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{
		if (!bomber_object_counter()->is_dead())
		{
			last_bomber = bomber_object_counter();
		}
	}

	if (last_bomber)
	{
		last_bomber->inc_points();
	}
	
	if (Config::get_rotate_levels())
	{
		app->map->load( CL_String("Levels/level") + ((Config::get_start_level()+rounds)%MAP_COUNT+1) );
	}
	else
	{
		app->map->load( CL_String("Levels/level") + Config::get_start_level() );
	}
	
	revive_bombers();
	clean_corpses();
	reset_round_time();
	game_status->analyze_game();
	game_status->show();
	Timer::reset();
	end_of_game = game_status->get_end_of_game(); //FIXME
	
	rounds++;
	
	
	CL_Vector pos;
	int c = 0;

	CL_Iterator<GameObject> object_counter2(app->objects);
	while (object_counter2.next() != NULL)
	{
		if ((object_counter2()->get_type() == GameObject::BOMB) || (object_counter2()->get_type() == GameObject::EXTRA))
		{
			delete object_counter2();
			object_counter2.remove();
		}
	}
	
	CL_Iterator<Bomber> bomber_object_counter2(app->bomber_objects);
	while (bomber_object_counter2.next() != NULL)
	{
		pos = app->map->get_bomber_pos(c++);
		bomber_object_counter2()->set_orig( pos.x*40, pos.y*40 );
		bomber_object_counter2()->init();
	}
	
	mode = NORMAL;
}

void Observer_Classic::clean_corpses()
{
	CL_Iterator<GameObject> object_counter(app->objects);
	while (object_counter.next() != NULL)
	{
		if (object_counter()->get_type() == GameObject::BOMBER_CORPSE)
		{
			delete object_counter();
			object_counter.remove();
		}
	}
}

void Observer_Classic::revive_bombers()
{
	CL_Iterator<Bomber> bomber_object_counter(corpse_container);
	while (bomber_object_counter.next() != NULL)
	{		
		app->bomber_objects.add(bomber_object_counter());
	}	
	corpse_container.empty_list();
}

void Observer_Classic::kill_all_bombers()
{
	CL_Iterator<Bomber> bomber_object_counter(app->bomber_objects);
	while (bomber_object_counter.next() != NULL)
	{		
		bomber_object_counter()->die();
	}	
}

void Observer_Classic::show()
{
	
	//app->fnt_clansoft->print_left( 600, 3, CL_String("Round: ") + (rounds+1) );
	if (round_time >0)
	{
		CL_Display::fill_rect( 625,3, 680,28, 0,0,0,1 );
		app->fnt_clansoft->print_left( 500, 3, CL_String("Time_Left: ") + ((int)round_time) );
	}
	else
	{
		CL_Display::fill_rect( 560,3, 680,28, 0,0,0,1 );
		app->fnt_clansoft->print_left( 500, 3, CL_String("Time up!!!!" ));
	}
	
	app->sur_observer->put_screen (15,20,0);
	
	if ((round_time < 21) && (round_time > 18))
	{
		if (fmod(round_time,0.3f) > 0.15f)
		{
			CL_Display::fill_rect( 300,250, 500,280, 0,0,0,0.5f );
			app->fnt_clansoft->print_left( 330, 254, "HURRY UP !!!! ");
			app->sur_observer->put_screen (260,245,0);
			app->sur_observer->put_screen (500,245,0);
		}
		if (play_hurryup)
		{			
			app->sfx_hurryup->play();
			play_hurryup = false;
		}
	}
}

void Observer_Classic::reset_round_time()
{
	round_time=Config::get_round_time();
	play_hurryup = true;
}





