/***************************************************************************
                          PlayerSetup.cpp  -  description                              
                             -------------------                                         
    begin                : Fri May 14 1999                                           
    copyright            : (C) 1999 by Andreas Hundt, Denis Oliver Kropp                         
    email                : clanbomber@fischlustig.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <ClanLib/clanlib.h>

#include "PlayerSetup.h"

#include "ClanBomber.h"
#include "Config.h"
#include "Timer.h"

PlayerSetup::PlayerSetup(ClanBomberApplication* _app)
{
	app = _app;
	cur_row = 0;
	cur_col = 0;
	key_buffer = CL_Input::keyboards[0]->get_buffer(0);
}

PlayerSetup::~PlayerSetup()
{
}

void PlayerSetup::run()
{
	draw();
	CL_Display::flip_display();	
	while (!CL_Keyboard::get_keycode(CL_Keyboard::KEY_ENTER))
	{
		CL_System::keep_alive();
	}
	key_buffer->clear();
	
	while (1)
	{
		while (key_buffer->keys_left() == 0)
		{
			CL_System::sleep( 50 );
			CL_System::keep_alive();
		}
		switch (key_buffer->get_key())
		{
			case CL_Keyboard::KEY_ESCAPE:
				return;
			break;
			case CL_Keyboard::KEY_ENTER:
				handle_enter();
				Config::save();
			break;
			case CL_Keyboard::KEY_SPACE:
				if (Config::bomber[cur_row].is_enabled())
				{
					Config::bomber[cur_row].disable();
				} else
				{
					Config::bomber[cur_row].enable();
				}
				Config::save();
			break;
			case CL_Keyboard::KEY_DOWN:
				cur_row += (cur_row<7) ? 1 : 0;
			break;
			case CL_Keyboard::KEY_UP:
				cur_row -= (cur_row>0) ? 1 : 0;
			break;
			case CL_Keyboard::KEY_LEFT:
				cur_col -= (cur_col>0) ? 1 : 0;
			break;
			case CL_Keyboard::KEY_RIGHT:
				cur_col += (cur_col<3) ? 1 : 0;
			break;
		}
		draw();
		CL_Display::flip_display();
	}
}

void PlayerSetup::draw(bool fick)
{
	int cg[5] = { 17, 95, 340, 500, 720 };

	CL_Display::clear_display();
	
	app->fnt_smallfont->print_center( 55, 35, "SKIN" );
	app->fnt_smallfont->print_left( 130, 35, "NAME" );
	app->fnt_smallfont->print_center( 418, 35, "TEAM" );
	app->fnt_smallfont->print_center( 612, 35, "CONTROLLER" );
	
	CL_Display::fill_rect( 0, 70+cur_row*63, 800, 70+cur_row*63 + 63,  0.2, 0.4, 0.8, 0.3 );
	if (!fick) CL_Display::fill_rect( cg[cur_col], 70+cur_row*63, cg[cur_col+1], 70+cur_row*63 + 63,  0.5, 0.5, 1.0, 0.4 );
	
	for (int i=0; i<8; i++)
	{
		app->sur_bomber[Config::bomber[i].get_skin()]->put_screen( 35, 70 + i*63, 0 );
		if (!fick || cur_row!=i) app->fnt_clansoft->print_left( 130, 90 + i*63, Config::bomber[i].get_name() );
		app->sur_ps_teams->put_screen( 360, 72 + i*63, Config::bomber[i].get_team() );
		app->sur_ps_controls->put_screen( 550, 72 + i*63, Config::bomber[i].get_controller() );
	
		if (!Config::bomber[i].is_enabled())
		{
			CL_Display::fill_rect( 0, 70+i*63, 800, 70+i*63 + 63,  0.1, 0.1, 0.1, 0.6 );
		}
	}
}

void PlayerSetup::handle_enter()
{
	switch (cur_col)
	{
		case 0:	Config::bomber[cur_row].set_skin( Config::bomber[cur_row].get_skin()+1 ); break;
		case 1:   enter_name() ; break;
//		case 2:	Config::bomber[cur_row].set_team( Config::bomber[cur_row].get_team()+1 ); break;
		case 3:	Config::bomber[cur_row].set_controller( Config::bomber[cur_row].get_controller()+1 ); break;
	}
}

void PlayerSetup::enter_name()
{
	float alpha = 0;
	
	while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ENTER))
	{
		CL_System::keep_alive();
	}
	
	key_buffer->clear();
	CL_String new_string = Config::bomber[cur_row].get_name();
	
	Timer::reset();
	while (1)
	{
		alpha += Timer::time_elapsed(true);
		while (alpha > 1) alpha -= 1;
		draw(true);
		CL_Display::fill_rect( 95,70+cur_row*63, 340,70+(cur_row+1)*63, 0.2f, 0.8f, 0.5f, (alpha>0.5f) ? (1-alpha)*2.0f : alpha*2.0f );

		app->fnt_clansoft->print_left( 130, 90 + cur_row*63, new_string );

		CL_Display::flip_display();
		
		CL_System::keep_alive();
		
		switch (key_buffer->get_key())
		{
			case CL_Keyboard::KEY_ENTER:
				Config::bomber[cur_row].set_name( new_string);
				return;
			break;
			case CL_Keyboard::KEY_BACKSPACE:
				new_string = new_string.mid( 0, new_string.get_length()-1 );
			break;
			case CL_Keyboard::KEY_SPACE:
				new_string += " ";
			break;
			case CL_Keyboard::KEY_A:
				new_string += "a";
			break;
			case CL_Keyboard::KEY_B:
				new_string += "b";
			break;
			case CL_Keyboard::KEY_C:
				new_string += "c";
			break;
			case CL_Keyboard::KEY_D:
				new_string += "d";
			break;
			case CL_Keyboard::KEY_E:
				new_string += "e";
			break;
			case CL_Keyboard::KEY_F:
				new_string += "f";
			break;
			case CL_Keyboard::KEY_G:
				new_string += "g";
			break;
			case CL_Keyboard::KEY_H:
				new_string += "h";
			break;
			case CL_Keyboard::KEY_I:
				new_string += "i";
			break;
			case CL_Keyboard::KEY_J:
				new_string += "j";
			break;
			case CL_Keyboard::KEY_K:
				new_string += "k";
			break;
			case CL_Keyboard::KEY_L:
				new_string += "l";
			break;
			case CL_Keyboard::KEY_M:
				new_string += "m";
			break;
			case CL_Keyboard::KEY_N:
				new_string += "n";
			break;
			case CL_Keyboard::KEY_O:
				new_string += "o";
			break;
			case CL_Keyboard::KEY_P:
				new_string += "p";
			break;
			case CL_Keyboard::KEY_Q:
				new_string += "q";
			break;
			case CL_Keyboard::KEY_R:
				new_string += "r";
			break;
			case CL_Keyboard::KEY_S:
				new_string += "s";
			break;
			case CL_Keyboard::KEY_T:
				new_string += "t";
			break;
			case CL_Keyboard::KEY_U:
				new_string += "u";
			break;
			case CL_Keyboard::KEY_V:
				new_string += "v";
			break;
			case CL_Keyboard::KEY_W:
				new_string += "w";
			break;
			case CL_Keyboard::KEY_X:
				new_string += "x";
			break;
			case CL_Keyboard::KEY_Y:
				new_string += "y";
			break;
			case CL_Keyboard::KEY_Z:
				new_string += "z";
			break;
		}
		if (new_string.get_length() == 1)
		{
			new_string.to_upper();
		}
	}
}







